# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import shutil
from datetime import datetime, timedelta
from azure.cli.testsdk import (StorageAccountPreparer, LiveScenarioTest, JMESPathCheck, ResourceGroupPreparer,
                               api_version_constraint)
from ..storage_test_util import StorageScenarioMixin, StorageTestFilesPreparer
from knack.util import CLIError


class StorageAzcopyTests(StorageScenarioMixin, LiveScenarioTest):

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    @StorageTestFilesPreparer()
    def test_storage_blob_azcopy_sync(self, resource_group, storage_account_info, test_dir):
        storage_account, _ = storage_account_info
        container = self.create_container(storage_account_info)

        # sync directory
        connection_string = self.cmd('storage account show-connection-string -n {} -g {} -otsv'
                                     .format(storage_account, resource_group)).output
        self.cmd('storage blob sync -s "{}" -c {} --connection-string {}'.format(
            test_dir, container, connection_string))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 41))

        self.cmd('storage blob delete-batch -s {} --account-name {}'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 0))

        # resync container
        self.cmd('storage blob sync -s "{}" -c {} --account-name {}'.format(
            test_dir, container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 41))

        # update file
        with open(os.path.join(test_dir, 'readme'), 'w') as f:
            f.write('updated.')
        # sync one blob
        self.cmd('storage blob list -c {} --account-name {} --prefix readme'.format(
            container, storage_account), checks=JMESPathCheck('[0].properties.contentLength', 87))
        self.cmd('storage blob sync -s "{}" -c {} --account-name {} -d readme'.format(
            os.path.join(test_dir, 'readme'), container, storage_account))
        self.cmd('storage blob list -c {} --account-name {} --prefix readme'.format(
            container, storage_account), checks=JMESPathCheck('[0].properties.contentLength', 8))

        # delete one file and sync
        os.remove(os.path.join(test_dir, 'readme'))
        self.cmd('storage blob sync -s "{}" -c {} --account-name {}'.format(
            test_dir, container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 40))

        # delete one folder and sync
        shutil.rmtree(os.path.join(test_dir, 'apple'))
        self.cmd('storage blob sync -s "{}" -c {} --account-name {}'.format(
            test_dir, container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 30))

        # sync with another folder
        self.cmd('storage blob sync -s "{}" -c {} --account-name {}'.format(
            os.path.join(test_dir, 'butter'), container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 20))

        # empty the folder and sync
        shutil.rmtree(os.path.join(test_dir, 'butter'))
        shutil.rmtree(os.path.join(test_dir, 'duff'))
        self.cmd('storage blob sync -s "{}" -c {} --account-name {}'.format(
            test_dir, container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 0))

        # sync a subset of files in a directory
        with open(os.path.join(test_dir, 'test.json'), 'w') as f:
            f.write('updated.')
        self.cmd('storage blob sync -s "{}" -c {} --account-name {} --include-pattern *.json'.format(
            test_dir, container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 1))

        self.cmd('storage blob delete-batch -s {} --account-name {}'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 0))

        # sync with guessing content-type
        cur = os.path.split(os.path.realpath(__file__))[0]
        source = os.path.join(cur, "testdir")
        self.storage_cmd('storage blob sync -s "{}" -c {} ', storage_account_info, source, container)

        self.storage_cmd('storage blob show -n "sample.js" -c {} ', storage_account_info, container)\
            .assert_with_checks(JMESPathCheck("name", "sample.js"),
                                JMESPathCheck("properties.contentSettings.contentType", "application/javascript"))

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    @StorageTestFilesPreparer()
    def test_storage_blob_azcopy_remove(self, resource_group, storage_account_info, test_dir):
        storage_account, _ = storage_account_info
        container = self.create_container(storage_account_info)

        # sync directory
        self.cmd('storage blob sync -s "{}" -c {} --account-name {}'.format(
            test_dir, container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 41))

        self.cmd('storage remove -c {} -n readme --account-name {}'.format(
            container, storage_account))

        self.cmd('storage remove -c {} -n readme --account-name {}'.format(
            container, storage_account))

        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 40))

        self.cmd('storage remove -c {} -n apple --account-name {}'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 30))

        self.cmd('storage remove -c {} -n butter --account-name {}'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 20))

        self.cmd('storage remove -c {} -n butter --account-name {} --recursive'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 10))

        # sync directory
        self.cmd('storage blob sync -s "{}" -c {} --account-name {}'.format(
            test_dir, container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 41))

        self.cmd('storage remove -c {} -n butter --account-name {} --exclude-pattern "file_1*"'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 32))

        self.cmd('storage remove -c {} -n butter --account-name {} --recursive --exclude-pattern "file_1*"'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 23))

        # sync directory
        self.cmd('storage blob sync -s "{}" -c {} --account-name {}'.format(
            test_dir, container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 41))

        self.cmd('storage remove -c {} -n butter --account-name {} --recursive --include-pattern "file_1*"'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 39))

        self.cmd('storage remove -c {} -n butter --account-name {} --include-pattern "file_*"'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 30))

        self.cmd('storage remove -c {} -n butter --account-name {} --recursive --include-pattern "file_*"'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 21))

        self.cmd('storage remove -c {} --include-path apple --account-name {} --include-pattern "file*" --exclude-pattern "file_1*" --recursive'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 12))

        self.cmd('storage remove -c {} --account-name {} --recursive'.format(
            container, storage_account))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            container, storage_account), checks=JMESPathCheck('length(@)', 0))

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_file_azcopy_remove(self, resource_group, storage_account):
        account_info = self.get_account_info(resource_group, storage_account)
        s1 = self.create_share(account_info)
        s2 = self.create_share(account_info)
        d1 = 'dir1'
        d2 = 'dir2'

        self.storage_cmd('storage directory create --share-name {} -n {}', account_info, s1, d1)
        self.storage_cmd('storage directory create --share-name {} -n {}', account_info, s2, d2)

        local_file = self.create_temp_file(512, full_random=False)
        src1_file = os.path.join(d1, 'source_file1.txt')
        src2_file = os.path.join(d2, 'source_file2.txt')

        self.storage_cmd('storage file upload -p "{}" --share-name {} --source "{}"', account_info,
                         src1_file, s1, local_file)
        self.storage_cmd('storage file exists -p "{}" -s {}', account_info, src1_file, s1) \
            .assert_with_checks(JMESPathCheck('exists', True))

        self.storage_cmd('storage remove --share-name {} -p "{}"',
                         account_info, s1, src1_file)
        self.storage_cmd('storage file exists -p "{}" -s {}', account_info, src1_file, s1) \
            .assert_with_checks(JMESPathCheck('exists', False))

        self.storage_cmd('storage file upload -p "{}" --share-name {} --source "{}"', account_info,
                         src2_file, s2, local_file)
        self.storage_cmd('storage file exists -p "{}" -s {}', account_info, src2_file, s2) \
            .assert_with_checks(JMESPathCheck('exists', True))

        self.storage_cmd('storage remove --share-name {} -p "{}"',
                         account_info, s2, d2)
        self.storage_cmd('storage file exists -p "{}" -s {}', account_info, src2_file, s2) \
            .assert_with_checks(JMESPathCheck('exists', False))

        self.storage_cmd('storage remove --share-name {} --recursive',
                         account_info, s2)
        self.storage_cmd('storage file list -s {}', account_info, s2) \
            .assert_with_checks(JMESPathCheck('length(@)', 0))

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='first_account')
    @StorageAccountPreparer(parameter_name='second_account', sku='Premium_LRS', kind='BlockBlobStorage')
    @StorageTestFilesPreparer()
    def test_storage_azcopy_blob_url(self, resource_group, first_account, second_account, test_dir):
        import os
        first_account_info = self.get_account_info(resource_group, first_account)
        second_account_info = self.get_account_info(resource_group, second_account)

        first_container = self.create_container(first_account_info)
        second_container = self.create_container(second_account_info)

        first_account_url = 'https://{}.blob.core.windows.net'.format(first_account)
        second_account_url = 'https://{}.blob.core.windows.net'.format(second_account)

        first_container_url = '{}/{}'.format(first_account_url, first_container)
        second_container_url = '{}/{}'.format(second_account_url, second_container)

        # test validation
        self.cmd('storage copy -s "{}" -d "{}" --destination-container test'
                 .format(os.path.join(test_dir, 'readme'), first_container_url), expect_failure=True)

        # Upload a single file
        content_type = "application/json"
        self.cmd('storage copy -s "{}" -d "{}" --content-type {}'.format(
            os.path.join(test_dir, 'readme'), first_container_url, content_type))
        self.storage_cmd('storage blob list -c {}', first_account_info, first_container)\
            .assert_with_checks(JMESPathCheck('length(@)', 1))
        self.storage_cmd('storage blob show -n {} -c {} ', first_account_info, 'readme', first_container)\
            .assert_with_checks(JMESPathCheck('properties.contentSettings.contentType', content_type))

        # Upload entire directory
        self.cmd('storage copy -s "{}" -d "{}" --recursive'.format(
            os.path.join(test_dir, 'apple'), first_container_url))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            first_container, first_account), checks=JMESPathCheck('length(@)', 11))

        # Upload a set of files
        self.cmd('storage copy -s "{}" -d "{}" --recursive'.format(
            os.path.join(test_dir, 'butter/file_*'), first_container_url))
        self.cmd('storage blob list -c {} --account-name {}'.format(
            first_container, first_account), checks=JMESPathCheck('length(@)', 21))

        local_folder = self.create_temp_dir()
        # Download a single file
        self.cmd('storage copy -s "{}" -d "{}"'.format(
            '{}/readme'.format(first_container_url), local_folder))
        self.assertEqual(1, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Download an entire directory
        self.cmd('storage copy -s "{}" -d "{}" --recursive'.format(
            '{}/apple'.format(first_container_url), local_folder))
        self.assertEqual(1, sum(len(d) for r, d, f in os.walk(local_folder)))
        self.assertEqual(11, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Download a set of files
        self.cmd('storage copy -s "{}" --include-path "apple" --include-pattern file* -d "{}" --recursive'.format(
            first_container_url, local_folder))
        self.assertEqual(3, sum(len(d) for r, d, f in os.walk(local_folder)))
        self.assertEqual(21, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Copy a single blob to another single blob
        self.cmd('storage account show -n {}'.format(second_account), checks=[
            self.check('kind', 'BlockBlobStorage')
        ])

        self.cmd('storage copy -s "{}" -d "{}" --preserve-s2s-access-tier false'.format(
            '{}/readme'.format(first_container_url), second_container_url))
        self.cmd('storage blob list -c {} --account-name {}'
                 .format(second_container, second_account), checks=JMESPathCheck('length(@)', 1))

        # Copy an entire directory from blob virtual directory to another blob virtual directory
        self.cmd('storage copy -s "{}" -d "{}" --recursive --preserve-s2s-access-tier false'.format(
            '{}/apple'.format(first_container_url), second_container_url))
        self.cmd('storage blob list -c {} --account-name {}'
                 .format(second_container, second_account), checks=JMESPathCheck('length(@)', 11))

        # Copy an entire storage account data to another blob account
        self.cmd('storage copy -s "{}" -d "{}" --recursive --preserve-s2s-access-tier false'.format(
            first_account_url, second_account_url))
        self.cmd('storage container list --account-name {}'
                 .format(second_account), checks=JMESPathCheck('length(@)', 2))
        self.cmd('storage blob list -c {} --account-name {}'
                 .format(first_container, second_account), checks=JMESPathCheck('length(@)', 21))

        # Upload to managed disk
        diskname = self.create_random_name(prefix='disk', length=12)
        local_file = self.create_temp_file(20480.5)
        self.cmd('disk create -n {} -g {} --for-upload --upload-size-bytes 20972032'
                 .format(diskname, resource_group))
        sasURL = self.cmd(
            'disk grant-access --access-level Write --duration-in-seconds 3600 -n {} -g {} --query accessSas -o tsv'
            .format(diskname, resource_group)).output.strip('\n')
        self.cmd('storage copy -s "{}" -d "{}" --blob-type PageBlob'
                 .format(local_file, sasURL))

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='first_account')
    @StorageAccountPreparer(parameter_name='second_account', sku='Premium_LRS', kind='BlockBlobStorage')
    @StorageTestFilesPreparer()
    def test_storage_azcopy_blob_account(self, resource_group, first_account, second_account, test_dir):

        first_account_info = self.get_account_info(resource_group, first_account)
        second_account_info = self.get_account_info(resource_group, second_account)

        first_container = self.create_container(first_account_info)
        second_container = self.create_container(second_account_info)

        first_account_url = 'https://{}.blob.core.windows.net'.format(first_account)

        import os
        # test validation
        self.cmd(
            'storage copy --source-local-path "{}" --destination-account-name {} --destination-container {} '
            '--destination {} '.format(
                os.path.join(test_dir, 'readme'), first_account, first_container, first_account_url),
            expect_failure=True)

        # Upload a single file
        self.cmd('storage copy --source-local-path "{}" --destination-account-name {} --destination-container {}'
                 .format(os.path.join(test_dir, 'readme'), first_account, first_container))
        self.cmd('storage blob list -c {} --account-name {}'
                 .format(first_container, first_account), checks=JMESPathCheck('length(@)', 1))

        # Upload entire directory
        self.cmd('storage copy --source-local-path "{}" --destination-account-name {} --destination-container {} --recursive'
                 .format(os.path.join(test_dir, 'apple'), first_account, first_container))
        self.cmd('storage blob list -c {} --account-name {}'
                 .format(first_container, first_account), checks=JMESPathCheck('length(@)', 11))

        # Upload a set of files
        self.cmd('storage copy --source-local-path "{}" --destination-account-name {} --destination-container {} --recursive'
                 .format(os.path.join(test_dir, 'butter/file_*'), first_account, first_container))
        self.cmd('storage blob list -c {} --account-name {}'
                 .format(first_container, first_account), checks=JMESPathCheck('length(@)', 21))

        # Upload a single file with a symlink
        source_path = os.path.join(test_dir, 'symlink_source')
        with open(source_path, 'w') as f:
            f.write('This is a data source for symlink.')
        symlink = os.path.join(test_dir, 'symlink')
        # If the error of "WinError[1314]" occurred during execution in Windows environment,
        # please try to execute azdev with administrator privileges
        os.symlink(source_path, symlink)

        self.cmd('storage copy --source-local-path "{}" --destination-account-name {} --destination-container {} '
                 '--follow-symlinks'.format(symlink, first_account, first_container))
        self.cmd('storage blob list -c {} --account-name {}'
                 .format(first_container, first_account), checks=JMESPathCheck('length(@)', 22))

        local_folder = self.create_temp_dir()
        # Download a single file
        self.cmd('storage copy --source-account-name {} --source-container {} --source-blob {} --destination-local-path "{}"'
                 .format(first_account, first_container, 'readme', local_folder))
        self.assertEqual(1, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Download entire directory
        self.cmd('storage copy --source-account-name {} --source-container {} --source-blob {} --destination-local-path "{}" --recursive'
                 .format(first_account, first_container, 'apple/', local_folder))
        self.assertEqual(1, sum(len(d) for r, d, f in os.walk(local_folder)))
        self.assertEqual(11, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Download a set of files
        self.cmd('storage copy --source-account-name {} --source-container {} --include-path {} --include-pattern {} --destination-local-path "{}" --recursive'
                 .format(first_account, first_container, 'apple', 'file*', local_folder))
        self.assertEqual(3, sum(len(d) for r, d, f in os.walk(local_folder)))
        self.assertEqual(21, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Copy a single blob to another single blob
        self.cmd('storage copy --source-account-name {} --source-container {} --source-blob {} \
                 --destination-account-name {} --destination-container {} --preserve-s2s-access-tier false'
                 .format(first_account, first_container, 'readme', second_account, second_container))
        self.cmd('storage blob list -c {} --account-name {}'
                 .format(second_container, second_account), checks=JMESPathCheck('length(@)', 1))

        # Copy an entire directory from blob virtual directory to another blob virtual directory
        self.cmd('storage copy --source-account-name {} --source-container {} --source-blob {} \
                 --destination-account-name {} --destination-container {} --recursive --preserve-s2s-access-tier false'
                 .format(first_account, first_container, 'apple', second_account, second_container))
        self.cmd('storage blob list -c {} --account-name {}'
                 .format(second_container, second_account), checks=JMESPathCheck('length(@)', 11))

        # Copy an entire storage account data to another blob account
        self.cmd('storage copy --source-account-name {} --destination-account-name {} --recursive --preserve-s2s-access-tier false'
                 .format(first_account, second_account))
        self.cmd('storage container list --account-name {}'
                 .format(second_account), checks=JMESPathCheck('length(@)', 2))
        self.cmd('storage blob list -c {} --account-name {}'
                 .format(first_container, second_account), checks=JMESPathCheck('length(@)', 22))

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='first_account')
    @StorageAccountPreparer(parameter_name='second_account', sku='Premium_LRS', kind='BlockBlobStorage')
    @StorageTestFilesPreparer()
    def test_storage_azcopy_blob_source_with_credential(self, resource_group, first_account, second_account, test_dir):

        first_account_info = self.get_account_info(resource_group, first_account)
        second_account_info = self.get_account_info(resource_group, second_account)

        first_container = self.create_container(first_account_info)
        second_container = self.create_container(second_account_info)

        import os
        # Upload a single file with destination account key
        self.storage_cmd('storage copy --source-local-path "{}" --destination-container {}', first_account_info,
                         os.path.join(test_dir, 'readme'), first_container)
        self.storage_cmd('storage blob list -c {} ', first_account_info, first_container, first_account)\
            .assert_with_checks(JMESPathCheck('length(@)', 1))

        # Upload entire directory with destination connection string
        first_connection_string = self.cmd('storage account show-connection-string -n {} -o tsv'.format(
            first_account)).output.strip()
        self.cmd('storage copy --source-local-path "{}" --destination-container {} --recursive --connection-string {}'
                 .format(os.path.join(test_dir, 'apple'), first_container, first_connection_string))
        self.storage_cmd('storage blob list -c {}', first_account_info, first_container)\
            .assert_with_checks(JMESPathCheck('length(@)', 11))

        # Download a single file with source account key
        local_folder = self.create_temp_dir()
        self.cmd('storage copy --source-account-name {} --source-account-key {} --source-container {} --source-blob {} '
                 '--destination-local-path "{}"'.format(first_account, first_account_info[1].strip(), first_container,
                                                        'readme', local_folder))
        self.assertEqual(1, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Download entire directory with source connection string
        self.cmd('storage copy --source-connection-string {} --source-container {} --source-blob {} '
                 '--destination-local-path "{}" --recursive'.format(first_connection_string, first_container,
                                                                    'apple/', local_folder))
        self.assertEqual(1, sum(len(d) for r, d, f in os.walk(local_folder)))
        self.assertEqual(11, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Download a set of files with source sas token
        expiry = (datetime.utcnow() + timedelta(hours=1)).strftime('%Y-%m-%dT%H:%MZ')
        first_sas_token = self.cmd(
            'storage container generate-sas --connection-string {} -n {} --expiry {} --permissions {} -o tsv'.format(
                first_connection_string, first_container, expiry, 'rwalcd')).output.strip()
        self.cmd('storage copy --source-account-name {} --source-sas {} --source-container {} --include-path {} '
                 '--include-pattern {} --destination-local-path "{}" --recursive'
                 .format(first_account, first_sas_token, first_container, 'apple', 'file*', local_folder))
        self.assertEqual(3, sum(len(d) for r, d, f in os.walk(local_folder)))
        self.assertEqual(21, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Copy a single blob to another single blob with sas token
        expiry = (datetime.utcnow() + timedelta(hours=1)).strftime('%Y-%m-%dT%H:%MZ')
        second_sas_token = self.storage_cmd(
            'storage container generate-sas -n {} --expiry {} --permissions {} -o tsv', second_account_info,
            second_container, expiry, 'acdlrw').output.strip()
        self.cmd('storage copy --source-account-name {} --source-container {} --source-blob {} --source-sas {} '
                 '--account-name {} --destination-container {} --sas-token {} '
                 '--preserve-s2s-access-tier false'
                 .format(first_account, first_container, 'readme', first_sas_token,
                         second_account, second_container, second_sas_token))
        self.storage_cmd('storage blob list -c {} ', second_account_info, second_container).assert_with_checks(
            JMESPathCheck('length(@)', 1))

        # Copy an entire storage account data to another blob account with account key
        self.storage_cmd('storage copy --source-account-name {} --source-account-key {} --recursive '
                         '--preserve-s2s-access-tier false', second_account_info, first_account,
                         first_account_info[1].strip())
        self.storage_cmd('storage container list ', second_account_info).assert_with_checks(
            JMESPathCheck('length(@)', 2))
        self.storage_cmd('storage blob list -c {} ', second_account_info, first_container)\
            .assert_with_checks(JMESPathCheck('length(@)', 11))

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    @StorageTestFilesPreparer()
    def test_storage_azcopy_file_url(self, resource_group, storage_account_info, test_dir):

        storage_account, _ = storage_account_info
        share = self.create_share(storage_account_info)

        storage_account_url = 'https://{}.file.core.windows.net'.format(storage_account)
        share_url = '{}/{}'.format(storage_account_url, share)

        import os
        # Upload a single file
        self.cmd('storage copy -s "{}" -d "{}"'
                 .format(os.path.join(test_dir, 'readme'), share_url))
        self.cmd('storage file list -s {} --account-name {}'
                 .format(share, storage_account), checks=JMESPathCheck('length(@)', 1))

        # Upload entire directory
        self.cmd('storage copy -s "{}" -d "{}" --recursive'
                 .format(os.path.join(test_dir, 'apple'), share_url))
        self.cmd('storage file list -s {} --account-name {}'.format(
            share, storage_account), checks=JMESPathCheck('length(@)', 2))

        # Upload a set of files
        self.cmd('storage copy -s "{}" -d "{}" --recursive'
                 .format(os.path.join(test_dir, 'butter/file_*'), share_url))
        self.cmd('storage file list -s {} --account-name {}'.format(
            share, storage_account), checks=JMESPathCheck('length(@)', 12))

        local_folder = self.create_temp_dir()
        # Download a single file
        self.cmd('storage copy -s "{}" -d "{}"'
                 .format('{}/readme'.format(share_url), local_folder))
        self.assertEqual(1, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Download entire directory
        self.cmd('storage copy -s "{}" -d "{}" --recursive'.format(
            '{}/apple'.format(share_url), local_folder))
        self.assertEqual(1, sum(len(d) for r, d, f in os.walk(local_folder)))
        self.assertEqual(11, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Download a set of files
        self.cmd('storage copy -s "{}" --include-path "apple" --include-pattern file* -d "{}" --recursive'.format(
            share_url, local_folder))
        self.assertEqual(3, sum(len(d) for r, d, f in os.walk(local_folder)))
        self.assertEqual(21, sum(len(f) for r, d, f in os.walk(local_folder)))

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    @StorageTestFilesPreparer()
    def test_storage_azcopy_file_account(self, resource_group, storage_account_info, test_dir):
        storage_account, _ = storage_account_info
        share = self.create_share(storage_account_info)

        import os
        # Upload a single file
        self.cmd('storage copy --source-local-path "{}" --destination-account-name {} --destination-share {}'
                 .format(os.path.join(test_dir, 'readme'), storage_account, share))
        self.cmd('storage file list -s {} --account-name {}'
                 .format(share, storage_account), checks=JMESPathCheck('length(@)', 1))

        # Upload entire directory
        self.cmd('storage copy --source-local-path "{}" --destination-account-name {} --destination-share {} --recursive'
                 .format(os.path.join(test_dir, 'apple'), storage_account, share))
        self.cmd('storage file list -s {} --account-name {}'.format(
            share, storage_account), checks=JMESPathCheck('length(@)', 2))

        # Upload a set of files
        self.cmd('storage copy --source-local-path "{}" --destination-account-name {} --destination-share {} --recursive'
                 .format(os.path.join(test_dir, 'butter/file_*'), storage_account, share))
        self.cmd('storage file list -s {} --account-name {}'.format(
            share, storage_account), checks=JMESPathCheck('length(@)', 12))

        local_folder = self.create_temp_dir()
        # Download a single file
        self.cmd('storage copy --source-account-name {} --source-share {} --source-file-path {} --destination-local-path "{}"'
                 .format(storage_account, share, 'readme', local_folder))
        self.assertEqual(1, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Download entire directory
        self.cmd('storage copy --source-account-name {} --source-share {} --source-file-path {} --destination-local-path "{}" --recursive'
                 .format(storage_account, share, 'apple', local_folder))
        self.assertEqual(1, sum(len(d) for r, d, f in os.walk(local_folder)))
        self.assertEqual(11, sum(len(f) for r, d, f in os.walk(local_folder)))

        # Download a set of files
        self.cmd('storage copy --source-account-name {} --source-share {} --include-path "apple" --include-pattern file* --destination-local-path "{}" --recursive'
                 .format(storage_account, share, local_folder))
        self.assertEqual(3, sum(len(d) for r, d, f in os.walk(local_folder)))
        self.assertEqual(21, sum(len(f) for r, d, f in os.walk(local_folder)))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
# pylint: disable=too-many-statements

from knack.arguments import CLIArgumentType

from azure.cli.core.commands.parameters import (
    get_resource_name_completion_list,
    tags_type, get_location_type,
    get_enum_type,
    resource_group_name_type,
    get_three_state_flag)
from azure.cli.command_modules.rdbms.validators import configuration_value_validator, validate_subnet, \
    tls_validator, public_access_validator, maintenance_window_validator, ip_address_validator, retention_validator
from azure.cli.core.local_context import LocalContextAttribute, LocalContextAction

from .randomname.generate import generate_username


def load_arguments(self, _):    # pylint: disable=too-many-statements

    server_completers = {
        'mariadb': get_resource_name_completion_list('Microsoft.DBForMariaDB/servers'),
        'mysql': get_resource_name_completion_list('Microsoft.DBForMySQL/servers'),
        'postgres': get_resource_name_completion_list('Microsoft.DBForPostgreSQL/servers')
    }

    def _complex_params(command_group):
        server_name_help = "Name of the server. The name can contain only lowercase letters, numbers, and the hyphen (-) character. " + \
                           "Minimum 3 characters and maximum 63 characters."
        server_name_scope = ['{}'.format(command_group)]
        server_name_setter_arg_type = CLIArgumentType(metavar='NAME', help=server_name_help, id_part='name',
                                                      local_context_attribute=LocalContextAttribute(name='server_name', actions=[LocalContextAction.SET], scopes=server_name_scope))
        server_name_getter_arg_type = CLIArgumentType(metavar='NAME', help=server_name_help, id_part='name',
                                                      local_context_attribute=LocalContextAttribute(name='server_name', actions=[LocalContextAction.GET], scopes=server_name_scope))
        server_name_arg_type = CLIArgumentType(metavar='NAME', help=server_name_help, id_part='name',
                                               local_context_attribute=LocalContextAttribute(name='server_name', actions=[LocalContextAction.SET, LocalContextAction.GET], scopes=server_name_scope))
        administrator_login_arg_type = CLIArgumentType(metavar='NAME',
                                                       local_context_attribute=LocalContextAttribute(name='administrator_login', actions=[LocalContextAction.GET, LocalContextAction.SET], scopes=server_name_scope))

        overriding_none_arg_type = CLIArgumentType(local_context_attribute=LocalContextAttribute(name='context_name', actions=[LocalContextAction.GET]))

        with self.argument_context(command_group) as c:
            c.argument('name', options_list=['--sku-name'], required=True)
            c.argument('server_name', arg_type=server_name_arg_type, completer=server_completers[command_group], options_list=['--server-name', '-s'])

        with self.argument_context('{} server'.format(command_group)) as c:
            c.ignore('family', 'capacity', 'tier')

            c.argument('server_name', arg_type=server_name_arg_type, options_list=['--name', '-n'], id_part='name', help='Name of the server. The name can contain only lowercase letters, numbers, and the hyphen (-) character. Minimum 3 characters and maximum 63 characters.')
            c.argument('administrator_login', options_list=['--admin-user', '-u'], help='Administrator username for the server. Once set, it cannot be changed.')
            c.argument('administrator_login_password', options_list=['--admin-password', '-p'], help='The password of the administrator. Minimum 8 characters and maximum 128 characters. Password must contain characters from three of the following categories: English uppercase letters, English lowercase letters, numbers, and non-alphanumeric characters.')
            c.argument('ssl_enforcement', arg_type=get_enum_type(['Enabled', 'Disabled']), options_list=['--ssl-enforcement'], help='Enable or disable ssl enforcement for connections to server. Default is Enabled.')
            c.argument('minimal_tls_version', arg_type=get_enum_type(['TLS1_0', 'TLS1_1', 'TLS1_2', 'TLSEnforcementDisabled']), options_list=['--minimal-tls-version'], help='Set the minimal TLS version for connections to server when SSL is enabled. Default is TLSEnforcementDisabled.', validator=tls_validator)
            c.argument('public_network_access', options_list=['--public-network-access', '--public'], help='Enable or disable public network access to server. When disabled, only connections made through Private Links can reach this server. Allowed values are : Enabled, Disabled, all, 0.0.0.0, <SingleIP>, <StartIP-DestinationIP>. Default is Enabled.')
            c.argument('tier', arg_type=get_enum_type(['Basic', 'GeneralPurpose', 'MemoryOptimized']), options_list=['--performance-tier'], help='The performance tier of the server.')
            c.argument('capacity', options_list=['--vcore'], type=int, help='Number of vcore.')
            c.argument('family', options_list=['--family'], arg_type=get_enum_type(['Gen4', 'Gen5']), help='Hardware generation.')
            c.argument('storage_mb', options_list=['--storage-size'], type=int, help='The storage capacity of the server (unit is megabytes). Minimum 5120 and increases in 1024 increments. Default is 51200.')
            c.argument('backup_retention', options_list=['--backup-retention'], type=int, help='The number of days a backup is retained. Range of 7 to 35 days. Default is 7 days.', validator=retention_validator)
            c.argument('auto_grow', arg_type=get_enum_type(['Enabled', 'Disabled']), options_list=['--auto-grow'], help='Enable or disable autogrow of the storage. Default value is Enabled.')
            c.argument('infrastructure_encryption', arg_type=get_enum_type(['Enabled', 'Disabled']), options_list=['--infrastructure-encryption', '-i'], help='Add an optional second layer of encryption for data using new encryption algorithm. Default value is Disabled.')
            c.argument('assign_identity', options_list=['--assign-identity'], help='Generate and assign an Azure Active Directory Identity for this server for use with key management services like Azure KeyVault.')
            c.argument('tags', tags_type)

            if command_group == 'mariadb':
                c.ignore('minimal_tls_version')
                c.ignore('assign_identity')
                c.ignore('infrastructure_encryption')

        with self.argument_context('{} server create'.format(command_group)) as c:
            c.argument('server_name', options_list=['--name', '-n'], arg_type=server_name_setter_arg_type)
            c.argument('sku_name', default='GP_Gen5_2', options_list=['--sku-name'],
                       help='The name of the sku. Follows the convention {pricing tier}_{compute generation}_{vCores} in shorthand. Examples: B_Gen5_1, GP_Gen5_4, MO_Gen5_16. ')
            c.argument('administrator_login', default=generate_username(), arg_group='Authentication')
            c.argument('administrator_login_password', arg_group='Authentication')

            c.argument('backup_retention', type=int, options_list=['--backup-retention'], help='The number of days a backup is retained. Range of 7 to 35 days. Default is 7 days.', validator=retention_validator)
            c.argument('geo_redundant_backup', arg_type=get_enum_type(['Enabled', 'Disabled']), options_list=['--geo-redundant-backup'], help='Enable or disable geo-redundant backups. Default value is Disabled. Not supported in Basic pricing tier.')
            c.argument('storage_mb', default=51200, options_list=['--storage-size'], type=int, help='The storage capacity of the server (unit is megabytes). Minimum 5120 and increases in 1024 increments. Default is 51200.')
            c.argument('auto_grow', arg_type=get_enum_type(['Enabled', 'Disabled']), options_list=['--auto-grow'], help='Enable or disable autogrow of the storage. Default value is Enabled.')
            c.argument('infrastructure_encryption', arg_type=get_enum_type(['Enabled', 'Disabled']), options_list=['--infrastructure-encryption', '-i'], help='Add an optional second layer of encryption for data using new encryption algorithm. Default value is Disabled.')
            c.argument('assign_identity', options_list=['--assign-identity'], help='Generate and assign an Azure Active Directory Identity for this server for use with key management services like Azure KeyVault.')

            c.argument('location', arg_type=get_location_type(self.cli_ctx))
            c.argument('version', help='Server major version.')

        with self.argument_context('{} server update'.format(command_group)) as c:
            c.ignore('family', 'capacity', 'tier')
            c.argument('sku_name', options_list=['--sku-name'], help='The name of the sku. Follows the convention {pricing tier}_{compute generation}_{vCores} in shorthand. Examples: B_Gen5_1, GP_Gen5_4, MO_Gen5_16.')
            c.argument('assign_identity', options_list=['--assign-identity'], help='Generate and assign an Azure Active Directory Identity for this server for use with key management services like Azure KeyVault.')

        with self.argument_context('{} server restore'. format(command_group)) as c:
            c.argument('server_name', options_list=['--name', '-n'], arg_type=overriding_none_arg_type)
            c.argument('source_server', options_list=['--source-server', '-s'], help='The name or resource ID of the source server to restore from.')
            c.argument('restore_point_in_time', options_list=['--restore-point-in-time', '--pitr-time'], help='The point in time to restore from (ISO8601 format), e.g., 2017-04-26T02:10:00+08:00')

        with self.argument_context('{} server georestore'. format(command_group)) as c:
            c.argument('location', arg_type=get_location_type(self.cli_ctx), required=True)
            c.argument('sku_name', options_list=['--sku-name'], required=False, help='The name of the sku. Defaults to sku of the source server. Follows the convention {pricing tier}_{compute generation}_{vCores} in shorthand. Examples: B_Gen5_1, GP_Gen5_4, MO_Gen5_16.')
            c.argument('source_server', options_list=['--source-server', '-s'], required=True, help='The name or ID of the source server to restore from.')
            c.argument('backup_retention', options_list=['--backup-retention'], type=int, help='The number of days a backup is retained. Range of 7 to 35 days. Default is 7 days.', validator=retention_validator)
            c.argument('geo_redundant_backup', options_list=['--geo-redundant-backup'], help='Enable or disable geo-redundant backups. Default value is Disabled. Not supported in Basic pricing tier.')

        with self.argument_context('{} server replica'.format(command_group)) as c:
            c.argument('server_name', options_list=['--name', '-n'], arg_type=overriding_none_arg_type)
            c.argument('source_server', options_list=['--source-server', '-s'], help='The name or resource ID of the master server to the create replica for.')
            c.argument('location', options_list=['--location', '-l'], help='Location. Values from: `az account list-locations`. If not provided, the create replica will be in the same location as the master server')
            c.argument('sku_name', options_list=['--sku-name'], help='The name of the sku. Follows the convention {pricing tier}_{compute generation}_{vCores} in shorthand. Examples: B_Gen5_1, GP_Gen5_4, MO_Gen5_16.')

        with self.argument_context('{} server configuration set'.format(command_group)) as c:
            c.argument('value', help='Value of the configuration. If not provided, configuration value will be set to default.', validator=configuration_value_validator)
            c.ignore('source')

        with self.argument_context('{} server wait'.format(command_group)) as c:
            c.ignore('created', 'deleted', 'updated')

        with self.argument_context('{} server delete'.format(command_group)) as c:
            c.argument('server_name', options_list=['--name', '-n'], arg_type=server_name_getter_arg_type)

        with self.argument_context('{} server-logs'.format(command_group)) as c:
            c.argument('file_name', options_list=['--name', '-n'], nargs='+', help='Space-separated list of log filenames on the server to download.')
            c.argument('max_file_size', type=int, help='The file size limitation to filter files.')
            c.argument('file_last_written', type=int, help='Integer in hours to indicate file last modify time, default value is 72.')
            c.argument('filename_contains', help='The pattern that file name should match.')

        with self.argument_context('{} server-logs list'.format(command_group)) as c:
            c.argument('server_name', id_part=None, help='Name of the Server.')

        with self.argument_context('{} db'.format(command_group)) as c:
            c.argument('database_name', options_list=['--name', '-n'])

        with self.argument_context('{} db list'.format(command_group)) as c:
            c.argument('server_name', id_part=None, help='Name of the Server.')

        with self.argument_context('{} server firewall-rule'.format(command_group)) as c:
            c.argument('server_name', options_list=['--server-name', '-s'])
            c.argument('firewall_rule_name', options_list=['--name', '-n'], id_part='child_name_1', help='The name of the firewall rule. The firewall rule name cannot be empty. The firewall rule name can only contain 0-9, a-z, A-Z, \'-\' and \'_\'. Additionally, the firewall rule name cannot exceed 128 characters.')
            c.argument('start_ip_address', options_list=['--start-ip-address'], help='The start IP address of the firewall rule. Must be IPv4 format. Use value \'0.0.0.0\' to represent all Azure-internal IP addresses.')
            c.argument('end_ip_address', options_list=['--end-ip-address'], help='The end IP address of the firewall rule. Must be IPv4 format. Use value \'0.0.0.0\' to represent all Azure-internal IP addresses.')

        with self.argument_context('{} server vnet-rule'.format(command_group)) as c:
            c.argument('server_name', options_list=['--server-name', '-s'])
            c.argument('virtual_network_rule_name', options_list=['--name', '-n'], id_part='child_name_1', help='The name of the vnet rule.')
            c.argument('virtual_network_subnet_id', options_list=['--subnet'], help='Name or ID of the subnet that allows access to an Azure Postgres Server. If subnet name is provided, --vnet-name must be provided.')
            c.argument('ignore_missing_vnet_service_endpoint', options_list=['--ignore-missing-endpoint', '-i'], help='Create vnet rule before virtual network has vnet service endpoint enabled', arg_type=get_three_state_flag())

        with self.argument_context('{} server vnet-rule create'.format(command_group)) as c:
            c.extra('vnet_name', options_list=['--vnet-name'], help='The virtual network name', validator=validate_subnet)

        with self.argument_context('{} server vnet-rule update'.format(command_group)) as c:
            c.extra('vnet_name', options_list=['--vnet-name'], help='The virtual network name',
                    validator=validate_subnet)

        with self.argument_context('{} server configuration'.format(command_group)) as c:
            c.argument('server_name', options_list=['--server-name', '-s'])
            c.argument('configuration_name', id_part='child_name_1', options_list=['--name', '-n'])

        with self.argument_context('{} server replica list'.format(command_group)) as c:
            c.argument('server_name', options_list=['--server-name', '-s'], help='Name of the master server.')

        for item in ['approve', 'reject', 'delete', 'show']:
            with self.argument_context('{} server private-endpoint-connection {}'.format(command_group, item)) as c:
                c.argument('private_endpoint_connection_name', options_list=['--name', '-n'], required=False,
                           help='The name of the private endpoint connection associated with the Server. '
                                'Required if --id is not specified')
                c.extra('connection_id', options_list=['--id'], required=False,
                        help='The ID of the private endpoint connection associated with the Server. '
                             'If specified --server-name/-s and --name/-n, this should be omitted.')
                c.argument('server_name', options_list=['--server-name', '-s'], required=False,
                           help='Name of the Server. Required if --id is not specified')
                c.argument('resource_group_name', help='The resource group name of specified server.',
                           required=False)
                c.argument('description', help='Comments for {} operation.'.format(item))

        with self.argument_context('{} server private-link-resource'.format(command_group)) as c:
            c.argument('server_name', options_list=['--server-name', '-s'], required=True, help='Name of the Server.')

        with self.argument_context('{} server list-skus'.format(command_group)) as c:
            c.argument('location_name', options_list=['--location', '-l'])

        with self.argument_context('{} server show-connection-string'.format(command_group)) as c:
            c.argument('server_name', options_list=['--server-name', '-s'], arg_type=server_name_arg_type, help='Name of the server.')
            c.argument('administrator_login', options_list=['--admin-user', '-u'], arg_type=administrator_login_arg_type,
                       help='The login username of the administrator.')
            c.argument('administrator_login_password', options_list=['--admin-password', '-p'],
                       help='The login password of the administrator.')
            c.argument('database_name', options_list=['--database-name', '-d'], help='The name of a database.')

        if command_group != 'mariadb':
            with self.argument_context('{} server key'.format(command_group)) as c:
                c.argument('server_name', options_list=['--name', '-s'])
                c.argument('kid', options_list=['--kid', '-k'], help='The Azure Key Vault key identifier of the server key. An example key identifier is "https://YourVaultName.vault.azure.net/keys/YourKeyName/01234567890123456789012345678901"')

            with self.argument_context('{} server ad-admin'.format(command_group)) as c:
                c.argument('server_name', options_list=['--server-name', '-s'])
                c.argument('login', options_list=['--display-name', '-u'], help='Display name of the Azure AD administrator user or group.')
                c.argument('sid', options_list=['--object-id', '-i'], help='The unique ID of the Azure AD administrator.')

        if command_group == 'mysql':
            with self.argument_context('{} server upgrade'.format(command_group)) as c:
                c.argument('target_server_version', options_list=['--target-server-version', '-t'], required=True, help='The server version you want to upgrade your mysql server to, currently only support 5.7.')

    _complex_params('mariadb')
    _complex_params('mysql')
    _complex_params('postgres')

    # Flexible-server
    def _flexible_server_params(command_group):

        server_name_setter_arg_type = CLIArgumentType(metavar='NAME',
                                                      help="Name of the server. The name can contain only lowercase letters, numbers, and the hyphen (-) character. Minimum 3 characters and maximum 63 characters.",
                                                      local_context_attribute=LocalContextAttribute(name='server_name', actions=[LocalContextAction.SET], scopes=['{} flexible-server'.format(command_group)]))

        server_name_getter_arg_type = CLIArgumentType(metavar='NAME',
                                                      help="Name of the server. The name can contain only lowercase letters, numbers, and the hyphen (-) character. Minimum 3 characters and maximum 63 characters.",
                                                      local_context_attribute=LocalContextAttribute(name='server_name', actions=[LocalContextAction.GET], scopes=['{} flexible-server'.format(command_group)]))

        server_name_arg_type = CLIArgumentType(metavar='NAME',
                                               help="Name of the server. The name can contain only lowercase letters, numbers, and the hyphen (-) character. Minimum 3 characters and maximum 63 characters.",
                                               local_context_attribute=LocalContextAttribute(name='server_name', actions=[LocalContextAction.SET, LocalContextAction.GET], scopes=['{} flexible-server'.format(command_group)]))

        administrator_login_setter_arg_type = CLIArgumentType(metavar='NAME',
                                                              local_context_attribute=LocalContextAttribute(name='administrator_login', actions=[LocalContextAction.SET], scopes=['{} flexible-server'.format(command_group)]))

        administrator_login_arg_type = CLIArgumentType(metavar='NAME',
                                                       local_context_attribute=LocalContextAttribute(name='administrator_login', actions=[LocalContextAction.GET, LocalContextAction.SET], scopes=['{} flexible-server'.format(command_group)]))

        database_name_setter_arg_type = CLIArgumentType(metavar='NAME',
                                                        local_context_attribute=LocalContextAttribute(name='database_name', actions=[LocalContextAction.SET], scopes=['{} flexible-server'.format(command_group)]))

        database_name_getter_arg_type = CLIArgumentType(metavar='NAME',
                                                        local_context_attribute=LocalContextAttribute(name='database_name', actions=[LocalContextAction.GET], scopes=['{} flexible-server'.format(command_group)]))

        database_name_arg_type = CLIArgumentType(metavar='NAME',
                                                 local_context_attribute=LocalContextAttribute(name='database_name', actions=[LocalContextAction.GET, LocalContextAction.SET], scopes=['{} flexible-server'.format(command_group)]))

        overriding_none_arg_type = CLIArgumentType(local_context_attribute=LocalContextAttribute(name='context_name', actions=[LocalContextAction.GET]))

        with self.argument_context('{} flexible-server'.format(command_group)) as c:
            c.argument('server_name', id_part='name', options_list=['--name', '-n'], arg_type=server_name_arg_type)

        with self.argument_context('{} flexible-server create'.format(command_group)) as c:
            # Add create mode as a parameter
            if command_group == 'postgres':
                c.argument('tier', default='GeneralPurpose', options_list=['--tier'],
                           help='Compute tier of the server. Accepted values: Burstable, GeneralPurpose, Memory Optimized ')
                c.argument('sku_name', default='Standard_D2s_v3', options_list=['--sku-name'],
                           help='The name of the compute SKU. Follows the convention Standard_{VM name}. Examples: Standard_D4s_v3 ')
                c.argument('storage_mb', default='128', options_list=['--storage-size'], type=int,
                           help='The storage capacity of the server. Minimum is 32 GiB and max is 16 TiB.')
                c.argument('backup_retention', default=7, type=int, options_list=['--backup-retention'],
                           help='The number of days a backup is retained. Range of 7 to 35 days. Default is 7 days.',
                           validator=retention_validator)
                c.argument('version', default='12', options_list=['--version'],
                           help='Server major version.')
                c.argument('zone', options_list=['--zone, -z'],
                           help='Availability zone into which to provision the resource.')
            elif command_group == 'mysql':
                c.argument('tier', default='Burstable',
                           help='Compute tier of the server. Accepted values: Burstable, GeneralPurpose, Memory Optimized ')
                c.argument('sku_name', default='Standard_B1ms', options_list=['--sku-name'],
                           help='The name of the compute SKU. Follows the convention Standard_{VM name}. Examples: Standard_B1ms, Standard_E16ds_v4 ')
                c.argument('storage_mb', default='10', options_list=['--storage-size'], type=int,
                           help='The storage capacity of the server. Minimum is 5 GiB and increases in 1 GiB increments. Max is 16 TiB.')
                c.argument('backup_retention', default=7, type=int, options_list=['--backup-retention'],
                           help='The number of days a backup is retained. Range of 7 to 35 days. Default is 7 days.')
                c.argument('version', default='5.7', options_list=['--version'],
                           help='Server major version.')
                c.argument('zone', options_list=['--zone, -z'],
                           help='Availability zone into which to provision the resource.')

            c.argument('vnet_resource_id', options_list=['--vnet'], help='Name of an existing virtual network or name of a new one to create. The name must be between 2 to 64 characters. The name must begin with a letter or number, end with a letter, number or underscore, and may contain only letters, numbers, underscores, periods, or hyphens.')
            c.argument('vnet_address_prefix', options_list=['--address-prefixes'], help='The IP address prefix to use when creating a new virtual network in CIDR format. Default value is 10.0.0.0/16.')
            c.argument('subnet_address_prefix', options_list=['--subnet-prefixes'], help='The subnet IP address prefix to use when creating a new VNet in CIDR format. Default value is 10.0.0.0/24.')
            c.argument('subnet_arm_resource_id', options_list=['--subnet'],
                       help='Resource ID of an existing subnet. Please note that the subnet will be delegated to Microsoft.DBforPostgreSQL/flexibleServers/Microsoft.DBforMySQL/flexibleServers.After delegation, this subnet cannot be used for any other type of Azure resources.')
            c.argument('server_name', options_list=['--name', '-n'], arg_type=server_name_setter_arg_type)
            c.argument('location', arg_type=get_location_type(self.cli_ctx))
            c.argument('administrator_login', default=generate_username(), options_list=['--admin-user, -u'], arg_group='Authentication', arg_type=administrator_login_setter_arg_type,
                       help='Administrator username for the server. Once set, it cannot be changed. ')
            c.argument('administrator_login_password', options_list=['--admin-password', '-p'],
                       help='The password of the administrator. Minimum 8 characters and maximum 128 characters. Password must contain characters from three of the following categories: English uppercase letters, English lowercase letters, numbers, and non-alphanumeric characters.',
                       arg_group='Authentication')
            c.argument('tags', tags_type)
            c.argument('public_access', options_list=['--public-access'],
                       help='Determines the public access. Enter single or range of IP addresses to be included in the allowed list of IPs. IP address ranges must be dash-separated and not contain any spaces. Specifying 0.0.0.0 allows public access from any resources deployed within Azure to access your server. Specifying no IP address sets the server in public access mode but does not create a firewall rule. ',
                       validator=public_access_validator)
            c.argument('high_availability', default="Disabled", options_list=['--high-availability'], help='Enable or disable high availability feature.  Default value is Disabled.')
            c.argument('assign_identity', options_list=['--assign-identity'],
                       help='Generate and assign an Azure Active Directory Identity for this server for use with key management services like Azure KeyVault. No need to enter extra argument.')
            c.ignore('database_name')

        with self.argument_context('{} flexible-server delete'.format(command_group)) as c:
            c.argument('resource_group_name', required=True)
            c.argument('server_name', id_part='name', options_list=['--name', '-n'], required=True, arg_type=server_name_getter_arg_type)
            c.argument('yes', options_list=['--yes', '-y'], action='store_true',
                       help='Do not prompt for confirmation.')

        with self.argument_context('{} flexible-server restore'.format(command_group)) as c:
            c.argument('server_name', options_list=['--name', '-n'], arg_type=overriding_none_arg_type,
                       help='The name of the new server that is created by the restore command.')
            c.argument('restore_point_in_time', options_list=['--restore-time'],
                       help='The point in time to restore from (ISO8601 format), e.g., 2017-04-26T02:10:00+08:00')
            if command_group == 'postgres':
                c.argument('source_server', options_list=['--source-server'],
                           help='The name of the source server to restore from.')
            elif command_group == 'mysql':
                c.argument('source_server', options_list=['--source-server'],
                           help='The name or resource ID of the source server to restore from.')

        with self.argument_context('{} flexible-server update'.format(command_group)) as c:
            c.ignore('assign_identity')
            c.argument('administrator_login_password', options_list=['--admin-password', '-p'],
                       help='The password of the administrator. Minimum 8 characters and maximum 128 characters. Password must contain characters from three of the following categories: English uppercase letters, English lowercase letters, numbers, and non-alphanumeric characters.',)
            c.argument('ha_enabled', options_list=['--high-availability'], arg_type=get_enum_type(['Enabled', 'Disabled']),
                       help='Enable or disable high availability feature.  Default value is Disabled.')
            c.argument('maintenance_window', options_list=['--maintenance-window'], validator=maintenance_window_validator,
                       help='Period of time (UTC) designated for maintenance. Examples: "Sun:23:30" to schedule on Sunday, 11:30pm UTC. To set back to default pass in "Disabled".')
            c.argument('tags', tags_type)
            if command_group == 'mysql':
                c.argument('tier', options_list=['--tier'],
                           help='Compute tier of the server. Accepted values: Burstable, GeneralPurpose, Memory Optimized')
                c.argument('sku_name', options_list=['--sku-name'],
                           help='The name of the compute SKU. Follows the convention Standard_{VM name}. Examples: Standard_B1ms, Standard_E16ds_v4 ')
                c.argument('storage_mb', options_list=['--storage-size'], type=int,
                           help='The storage capacity of the server. Minimum is 5 GiB and increases in 1 GiB increments. Max is 16 TiB.')
                c.argument('backup_retention', type=int, options_list=['--backup-retention'],
                           help='The number of days a backup is retained. Range of 7 to 35 days. Default is 7 days.')
                c.argument('auto_grow', arg_type=get_enum_type(['Enabled', 'Disabled']), options_list=['--storage-auto-grow'],
                           help='Enable or disable autogrow of the storage. Default value is Enabled.')
                c.argument('ssl_enforcement', arg_type=get_enum_type(['Enabled', 'Disabled']),
                           options_list=['--ssl-enforcement'],
                           help='Enable or disable ssl enforcement for connections to server. Default is Enabled.')
                c.argument('subnet_arm_resource_id', options_list=['--subnet'],
                           help='Name or ID of the subnet that allows access to an Azure Flexible Server MySQL Server. ')
                c.argument('replication_role', options_list=['--replication-role'],
                           help='The replication role of the server.')
            elif command_group == 'postgres':
                c.argument('tier', options_list=['--tier'],
                           help='Compute tier of the server. Accepted values: Burstable, GeneralPurpose, Memory Optimized')
                c.argument('sku_name', options_list=['--sku-name'],
                           help='The name of the compute SKU. Follows the convention Standard_{VM name}. Examples: Standard_D4s_v3 ')
                c.argument('storage_mb', options_list=['--storage-size'], type=int,
                           help='The storage capacity of the server. Minimum is 32 GiB and max is 16 TiB.')
                c.argument('backup_retention', type=int, options_list=['--backup-retention'],
                           help='The number of days a backup is retained. Range of 7 to 35 days. Default is 7 days.', validator=retention_validator)

        with self.argument_context('{} flexible-server list-skus'.format(command_group)) as c:
            c.argument('location', arg_type=get_location_type(self.cli_ctx))

        # flexible-server parameter
        for scope in ['list', 'set', 'show']:
            argument_context_string = '{} flexible-server parameter {}'.format(command_group, scope)
            with self.argument_context(argument_context_string) as c:
                if scope == "list":
                    c.argument('server_name', id_part=None, options_list=['--server-name', '-s'], arg_type=server_name_arg_type)
                else:
                    c.argument('server_name', id_part='name', options_list=['--server-name', '-s'],
                               arg_type=server_name_arg_type)

        for scope in ['show', 'set']:
            argument_context_string = '{} flexible-server parameter {}'.format(command_group, scope)
            with self.argument_context(argument_context_string) as c:
                c.argument('configuration_name', id_part='child_name_1', options_list=['--name', '-n'], required=True,
                           help='The name of the server configuration')

        with self.argument_context('{} flexible-server parameter set'.format(command_group)) as c:
            c.argument('value', options_list=['--value', '-v'],
                       help='Value of the configuration.')
            c.argument('source', options_list=['--source'],
                       help='Source of the configuration.')

        # firewall-rule
        for scope in ['create', 'delete', 'list', 'show', 'update']:
            argument_context_string = '{} flexible-server firewall-rule {}'.format(command_group, scope)
            with self.argument_context(argument_context_string) as c:
                c.argument('resource_group_name', arg_type=resource_group_name_type)
                if scope == "list":
                    c.argument('server_name', id_part=None, options_list=['--name', '-n'], arg_type=server_name_arg_type)
                else:
                    c.argument('server_name', id_part='name', options_list=['--name', '-n'], arg_type=server_name_arg_type)

        for scope in ['create', 'delete', 'show', 'update']:
            argument_context_string = '{} flexible-server firewall-rule {}'.format(command_group, scope)
            with self.argument_context(argument_context_string) as c:
                c.argument('firewall_rule_name', id_part='child_name_1', options_list=['--rule-name', '-r'],
                           help='The name of the firewall rule. If name is omitted, default name will be chosen for firewall name. The firewall rule name can only contain 0-9, a-z, A-Z, \'-\' and \'_\'. Additionally, the firewall rule name cannot exceed 128 characters. ')

        with self.argument_context('{} flexible-server firewall-rule create'.format(command_group)) as c:
            c.argument('end_ip_address', options_list=['--end-ip-address'], validator=ip_address_validator,
                       help='The end IP address of the firewall rule. Must be IPv4 format. Use value \'0.0.0.0\' to represent all Azure-internal IP addresses. ')
            c.argument('start_ip_address', options_list=['--start-ip-address'], validator=ip_address_validator,
                       help='The start IP address of the firewall rule. Must be IPv4 format. Use value \'0.0.0.0\' to represent all Azure-internal IP addresses. ')

        with self.argument_context('{} flexible-server firewall-rule delete'.format(command_group)) as c:
            c.argument('yes', options_list=['--yes', '-y'], action='store_true', help='Do not prompt for confirmation.')

        with self.argument_context('{} flexible-server firewall-rule update'.format(command_group)) as c:
            c.argument('end_ip_address', options_list=['--end-ip-address'], validator=ip_address_validator,
                       help='When using \'set\' or \'add\', preserve string literals instead of attempting to convert to JSON.')
            c.argument('start_ip_address', options_list=['--start-ip-address'], validator=ip_address_validator,
                       help='The start IP address of the firewall rule. Must be IPv4 format. Use value \'0.0.0.0\' to represent all Azure-internal IP addresses. ')

        # db
        if command_group == "mysql":
            with self.argument_context('{} flexible-server db'.format(command_group)) as c:
                c.argument('server_name', options_list=['--server-name', '-s'], help='Name of the server.')
                c.argument('database_name', arg_type=database_name_arg_type, options_list=['--database-name', '-d'], help='The name of a database.')

            with self.argument_context('{} flexible-server db create'.format(command_group)) as c:
                c.argument('database_name', arg_type=database_name_setter_arg_type, options_list=['--database-name', '-d'], help='The name of a database.')

            with self.argument_context('{} flexible-server db list'.format(command_group)) as c:
                c.argument('server_name', id_part=None, options_list=['--server-name', '-s'], arg_type=server_name_arg_type)
                c.argument('database_name', id_part=None, arg_type=database_name_setter_arg_type, options_list=['--database-name', '-d'], help='The name of a database.')

            with self.argument_context('{} flexible-server db delete'.format(command_group)) as c:
                c.argument('database_name', arg_type=database_name_getter_arg_type, options_list=['--database-name', '-d'], help='The name of a database.')
                c.argument('yes', options_list=['--yes', '-y'], action='store_true', help='Do not prompt for confirmation.')

        with self.argument_context('{} flexible-server show-connection-string'.format(command_group)) as c:
            c.argument('server_name', options_list=['--server-name', '-s'], arg_type=server_name_arg_type, help='Name of the server.')
            c.argument('administrator_login', arg_type=administrator_login_arg_type, options_list=['--admin-user', '-u'],
                       help='The login username of the administrator.')
            c.argument('administrator_login_password', options_list=['--admin-password', '-p'],
                       help='The login password of the administrator.')
            c.argument('database_name', arg_type=database_name_arg_type, options_list=['--database-name', '-d'], help='The name of a database.')

        with self.argument_context('{} flexible-server replica list'.format(command_group)) as c:
            c.argument('server_name', id_part=None, options_list=['--name', '-n'], help='Name of the source server.')

        with self.argument_context('{} flexible-server replica create'.format(command_group)) as c:
            c.argument('server_name', options_list=['--source-server'],
                       help='The name or resource ID of the source server to restore from.')
            c.argument('replica_name', options_list=['--replica-name'],
                       help='The name of the server to restore to.')
            c.ignore('location')
            c.ignore('sku_name')
            c.ignore('tier')

        with self.argument_context('{} flexible-server replica stop-replication'.format(command_group)) as c:
            c.argument('server_name', options_list=['--name', '-n'], help='Name of the replica server.')

    _flexible_server_params('postgres')
    _flexible_server_params('mysql')



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=line-too-long
# pylint: disable=too-many-lines
# pylint: disable=no-else-return

from azure.cli.core.util import sdk_no_wait


def billing_account_list(client,
                         expand=None):
    return client.list(expand=expand)


def billing_account_show(client,
                         account_name,
                         expand=None):
    return client.get(billing_account_name=account_name,
                      expand=expand)


def billing_account_update(client,
                           account_name,
                           display_name=None,
                           sold_to=None,
                           departments=None,
                           enrollment_accounts=None,
                           billing_profiles_value=None,
                           no_wait=False):
    parameters = {}
    parameters['display_name'] = display_name
    parameters['sold_to'] = sold_to
    parameters['departments'] = departments
    parameters['enrollment_accounts'] = enrollment_accounts
    parameters['billing_profiles'] = {}
    parameters['billing_profiles']['value'] = billing_profiles_value
    return sdk_no_wait(no_wait,
                       client.update,
                       billing_account_name=account_name,
                       parameters=parameters)


def billing_balance_show(client,
                         account_name,
                         profile_name):
    return client.get(billing_account_name=account_name,
                      billing_profile_name=profile_name)


def billing_instruction_list(client,
                             account_name,
                             profile_name):
    return client.list_by_billing_profile(billing_account_name=account_name,
                                          billing_profile_name=profile_name)


def billing_instruction_show(client,
                             account_name,
                             profile_name,
                             name):
    return client.get(billing_account_name=account_name,
                      billing_profile_name=profile_name,
                      instruction_name=name)


def billing_instruction_create(client,
                               account_name,
                               profile_name,
                               name,
                               amount=None,
                               start_date=None,
                               end_date=None,
                               creation_date=None):
    parameters = {}
    parameters['amount'] = amount
    parameters['start_date'] = start_date
    parameters['end_date'] = end_date
    parameters['creation_date'] = creation_date
    return client.put(billing_account_name=account_name,
                      billing_profile_name=profile_name,
                      instruction_name=name,
                      parameters=parameters)


def billing_profile_list(client,
                         account_name,
                         expand=None):
    return client.list_by_billing_account(billing_account_name=account_name,
                                          expand=expand)


def billing_profile_show(client,
                         account_name,
                         profile_name,
                         expand=None):
    return client.get(billing_account_name=account_name,
                      billing_profile_name=profile_name,
                      expand=expand)


def billing_profile_create(client,
                           account_name,
                           profile_name,
                           display_name=None,
                           po_number=None,
                           bill_to=None,
                           invoice_email_opt_in=None,
                           enabled_azure_plans=None,
                           invoice_sections_value=None,
                           no_wait=False):
    parameters = {}
    parameters['display_name'] = display_name
    parameters['po_number'] = po_number
    parameters['bill_to'] = bill_to
    parameters['invoice_email_opt_in'] = invoice_email_opt_in
    parameters['enabled_azure_plans'] = enabled_azure_plans
    parameters['invoice_sections'] = {}
    parameters['invoice_sections']['value'] = invoice_sections_value
    return sdk_no_wait(no_wait,
                       client.create_or_update,
                       billing_account_name=account_name,
                       billing_profile_name=profile_name,
                       parameters=parameters)


def billing_profile_update(client,
                           account_name,
                           profile_name,
                           display_name=None,
                           po_number=None,
                           bill_to=None,
                           invoice_email_opt_in=None,
                           enabled_azure_plans=None,
                           invoice_sections_value=None,
                           no_wait=False):
    parameters = {}
    parameters['display_name'] = display_name
    parameters['po_number'] = po_number
    parameters['bill_to'] = bill_to
    parameters['invoice_email_opt_in'] = invoice_email_opt_in
    parameters['enabled_azure_plans'] = enabled_azure_plans
    parameters['invoice_sections'] = {}
    parameters['invoice_sections']['value'] = invoice_sections_value
    return sdk_no_wait(no_wait,
                       client.create_or_update,
                       billing_account_name=account_name,
                       billing_profile_name=profile_name,
                       parameters=parameters)


def billing_customer_list(client,
                          account_name,
                          profile_name=None,
                          search=None,
                          filter_=None):
    if account_name is not None and profile_name is not None:
        return client.list_by_billing_profile(billing_account_name=account_name,
                                              billing_profile_name=profile_name,
                                              search=search,
                                              filter=filter_)
    return client.list_by_billing_account(billing_account_name=account_name,
                                          search=search,
                                          filter=filter_)


def billing_customer_show(client,
                          account_name,
                          customer_name,
                          expand=None):
    return client.get(billing_account_name=account_name,
                      customer_name=customer_name,
                      expand=expand)


def billing_invoice_section_list(client,
                                 account_name,
                                 profile_name):
    return client.list_by_billing_profile(billing_account_name=account_name,
                                          billing_profile_name=profile_name)


def billing_invoice_section_show(client,
                                 account_name,
                                 profile_name,
                                 invoice_section_name):
    return client.get(billing_account_name=account_name,
                      billing_profile_name=profile_name,
                      invoice_section_name=invoice_section_name)


def billing_invoice_section_create(client,
                                   account_name,
                                   profile_name,
                                   invoice_section_name,
                                   display_name=None,
                                   labels=None,
                                   no_wait=False):
    return sdk_no_wait(no_wait,
                       client.create_or_update,
                       billing_account_name=account_name,
                       billing_profile_name=profile_name,
                       invoice_section_name=invoice_section_name,
                       display_name=display_name,
                       labels=labels)


def billing_invoice_section_update(client,
                                   account_name,
                                   profile_name,
                                   invoice_section_name,
                                   display_name=None,
                                   labels=None,
                                   no_wait=False):
    return sdk_no_wait(no_wait,
                       client.create_or_update,
                       billing_account_name=account_name,
                       billing_profile_name=profile_name,
                       invoice_section_name=invoice_section_name,
                       display_name=display_name,
                       labels=labels)


def billing_permission_list(client,
                            account_name,
                            profile_name=None,
                            invoice_section_name=None,
                            customer_name=None):
    if account_name is not None and profile_name is not None and invoice_section_name is not None:
        return client.list_by_invoice_sections(billing_account_name=account_name,
                                               billing_profile_name=profile_name,
                                               invoice_section_name=invoice_section_name)
    elif account_name is not None and customer_name is not None:
        return client.list_by_customer(billing_account_name=account_name,
                                       customer_name=customer_name)
    elif account_name is not None and profile_name is not None:
        return client.list_by_billing_profile(billing_account_name=account_name,
                                              billing_profile_name=profile_name)
    return client.list_by_billing_account(billing_account_name=account_name)


def billing_subscription_list(client,
                              account_name,
                              profile_name=None,
                              invoice_section_name=None,
                              customer_name=None):
    if account_name is not None and profile_name is not None and invoice_section_name is not None:
        return client.list_by_invoice_section(billing_account_name=account_name,
                                              billing_profile_name=profile_name,
                                              invoice_section_name=invoice_section_name)
    elif account_name is not None and customer_name is not None:
        return client.list_by_customer(billing_account_name=account_name,
                                       customer_name=customer_name)
    elif account_name is not None and profile_name is not None:
        return client.list_by_billing_profile(billing_account_name=account_name,
                                              billing_profile_name=profile_name)
    return client.list_by_billing_account(billing_account_name=account_name)


def billing_subscription_show(client,
                              account_name):
    return client.get(billing_account_name=account_name)


def billing_subscription_update(client,
                                account_name,
                                subscription_billing_status=None,
                                cost_center=None,
                                sku_id=None):
    parameters = {}
    parameters['subscription_billing_status'] = subscription_billing_status
    parameters['cost_center'] = cost_center
    parameters['sku_id'] = sku_id
    return client.update(billing_account_name=account_name,
                         parameters=parameters)


def billing_subscription_move(client,
                              account_name,
                              destination_invoice_section_id,
                              no_wait=False):
    return sdk_no_wait(no_wait,
                       client.move,
                       billing_account_name=account_name,
                       destination_invoice_section_id=destination_invoice_section_id)


def billing_subscription_validate_move(client,
                                       account_name,
                                       destination_invoice_section_id):
    return client.validate_move(billing_account_name=account_name,
                                destination_invoice_section_id=destination_invoice_section_id)


def billing_product_list(client,
                         account_name,
                         profile_name=None,
                         invoice_section_name=None,
                         filter_=None,
                         customer_name=None):
    if account_name is not None and profile_name is not None and invoice_section_name is not None:
        return client.list_by_invoice_section(billing_account_name=account_name,
                                              billing_profile_name=profile_name,
                                              invoice_section_name=invoice_section_name,
                                              filter=filter_)
    elif account_name is not None and profile_name is not None:
        return client.list_by_billing_profile(billing_account_name=account_name,
                                              billing_profile_name=profile_name,
                                              filter=filter_)
    elif account_name is not None and customer_name is not None:
        return client.list_by_customer(billing_account_name=account_name,
                                       customer_name=customer_name)
    return client.list_by_billing_account(billing_account_name=account_name,
                                          filter=filter_)


def billing_product_show(client,
                         account_name,
                         product_name):
    return client.get(billing_account_name=account_name,
                      product_name=product_name)


def billing_product_update(client,
                           account_name,
                           product_name,
                           auto_renew=None,
                           status=None,
                           billing_frequency=None):
    parameters = {}
    parameters['auto_renew'] = auto_renew
    parameters['status'] = status
    parameters['billing_frequency'] = billing_frequency
    return client.update(billing_account_name=account_name,
                         product_name=product_name,
                         parameters=parameters)


def billing_product_move(client,
                         account_name,
                         product_name,
                         destination_invoice_section_id=None):
    return client.move(billing_account_name=account_name,
                       product_name=product_name,
                       destination_invoice_section_id=destination_invoice_section_id)


def billing_product_validate_move(client,
                                  account_name,
                                  product_name,
                                  destination_invoice_section_id=None):
    return client.validate_move(billing_account_name=account_name,
                                product_name=product_name,
                                destination_invoice_section_id=destination_invoice_section_id)


def billing_invoice_list(client,
                         period_start_date,
                         period_end_date,
                         account_name=None,
                         profile_name=None):
    if account_name is not None and profile_name is not None and period_start_date is not None and period_end_date is not None:
        return client.list_by_billing_profile(billing_account_name=account_name,
                                              billing_profile_name=profile_name,
                                              period_start_date=period_start_date,
                                              period_end_date=period_end_date)
    elif account_name is not None and period_start_date is not None and period_end_date is not None:
        return client.list_by_billing_account(billing_account_name=account_name,
                                              period_start_date=period_start_date,
                                              period_end_date=period_end_date)
    return client.list_by_billing_subscription(period_start_date=period_start_date,
                                               period_end_date=period_end_date)


def billing_invoice_show(client,
                         account_name,
                         name):
    return client.get(billing_account_name=account_name,
                      invoice_name=name)


def billing_transaction_list(client,
                             account_name,
                             invoice_name):
    return client.list_by_invoice(billing_account_name=account_name,
                                  invoice_name=invoice_name)


def billing_policy_update(client,
                          account_name,
                          profile_name,
                          marketplace_purchases=None,
                          reservation_purchases=None,
                          view_charges=None):
    parameters = {}
    parameters['marketplace_purchases'] = marketplace_purchases
    parameters['reservation_purchases'] = reservation_purchases
    parameters['view_charges'] = view_charges
    return client.update(billing_account_name=account_name,
                         billing_profile_name=profile_name,
                         parameters=parameters)


def billing_property_show(client):
    return client.get()


def billing_property_update(client,
                            cost_center=None):
    return client.update(cost_center=cost_center)


def billing_role_definition_list(client,
                                 account_name,
                                 profile_name=None,
                                 invoice_section_name=None):
    if account_name is not None and profile_name is not None and invoice_section_name is not None:
        return client.list_by_invoice_section(billing_account_name=account_name,
                                              billing_profile_name=profile_name,
                                              invoice_section_name=invoice_section_name)
    elif account_name is not None and profile_name is not None:
        return client.list_by_billing_profile(billing_account_name=account_name,
                                              billing_profile_name=profile_name)
    return client.list_by_billing_account(billing_account_name=account_name)


def billing_role_assignment_list(client,
                                 account_name,
                                 profile_name=None,
                                 invoice_section_name=None):
    if account_name is not None and profile_name is not None and invoice_section_name is not None:
        return client.list_by_invoice_section(billing_account_name=account_name,
                                              billing_profile_name=profile_name,
                                              invoice_section_name=invoice_section_name)
    elif account_name is not None and profile_name is not None:
        return client.list_by_billing_profile(billing_account_name=account_name,
                                              billing_profile_name=profile_name)
    return client.list_by_billing_account(billing_account_name=account_name)


def billing_role_assignment_delete(client,
                                   account_name,
                                   name,
                                   profile_name=None,
                                   invoice_section_name=None):
    if account_name is not None and profile_name is not None and invoice_section_name is not None and name is not None:
        return client.delete_by_invoice_section(billing_account_name=account_name,
                                                billing_profile_name=profile_name,
                                                invoice_section_name=invoice_section_name,
                                                billing_role_assignment_name=name)
    elif account_name is not None and profile_name is not None and name is not None:
        return client.delete_by_billing_profile(billing_account_name=account_name,
                                                billing_profile_name=profile_name,
                                                billing_role_assignment_name=name)
    return client.delete_by_billing_account(billing_account_name=account_name,
                                            billing_role_assignment_name=name)


def billing_agreement_list(client,
                           account_name,
                           expand=None):
    return client.list_by_billing_account(billing_account_name=account_name,
                                          expand=expand)


def billing_agreement_show(client,
                           account_name,
                           name,
                           expand=None):
    return client.get(billing_account_name=account_name,
                      agreement_name=name,
                      expand=expand)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest
import json
import logging
import os
import time
from azure.cli.command_modules.servicefabric.tests.latest.test_util import _create_cluster, _create_cluster_with_separate_kv, _wait_for_cluster_state_ready, _add_selfsigned_cert_to_keyvault
from azure.cli.core.util import CLIError
from azure.cli.testsdk import ScenarioTest, LiveScenarioTest, ResourceGroupPreparer

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))


class ServiceFabricClusterTests(ScenarioTest):
    @unittest.skip('Update api version')
    @ResourceGroupPreparer()
    def test_cluster_certs(self):
        self.kwargs.update({
            'kv_name': self.create_random_name('sfrp-cli-kv-', 24),
            'loc': 'westus',
            'cert_name': self.create_random_name('sfrp-cli-', 24),
            'cert_name2': self.create_random_name('sfrp-cli-cert2', 24),
            'cluster_name': self.create_random_name('sfrp-cli-', 24),
            'vm_password': "Pass123!@#",
            'policy_path': os.path.join(TEST_DIR, 'policy.json')
        })
        _create_cluster_with_separate_kv(self, self.kwargs)
        _wait_for_cluster_state_ready(self, self.kwargs)

        primary_cert = self.cmd('keyvault certificate show --vault-name {kv_name} -n {cert_name}').get_output_in_json()
        primary_cert_tp = primary_cert['x509ThumbprintHex']
        self.kwargs.update({'primary_cert_tp': primary_cert_tp})

        # add secondary cert
        self.cmd('keyvault certificate create --vault-name {kv_name} -n {cert_name2} -p @"{policy_path}"')

        while True:
            cert = self.cmd('keyvault certificate show --vault-name {kv_name} -n {cert_name}').get_output_in_json()
            if cert:
                break
        assert cert['sid'] is not None
        secondary_cert = cert
        secondary_cert_secret_id = secondary_cert['sid']
        secondary_cert_tp = secondary_cert['x509ThumbprintHex']
        self.kwargs.update({'secondary_cert_secret_id': secondary_cert_secret_id})
        self.cmd('sf cluster certificate add --resource-group {rg} --cluster-name {cluster_name} --secret-identifier {secondary_cert_secret_id}',
                 checks=[self.check('certificate.thumbprintSecondary', secondary_cert_tp)])

        _wait_for_cluster_state_ready(self, self.kwargs)

        # remove primary cert
        self.cmd('sf cluster certificate remove --resource-group {rg} --cluster-name {cluster_name} --thumbprint {primary_cert_tp}',
                 checks=[self.check('certificate.thumbprint', secondary_cert_tp)])

        _wait_for_cluster_state_ready(self, self.kwargs)

    @unittest.skip('no quota, disable temporarily')
    @ResourceGroupPreparer()
    def test_cluster(self):
        self.kwargs.update({
            'kv_name': self.create_random_name('sfrp-cli-kv-', 24),
            'loc': 'westus',
            'cert_name': self.create_random_name('sfrp-cli-', 24),
            'cluster_name': self.create_random_name('sfrp-cli-', 24),
            'vm_password': "Pass123!@#",
            'priamry_node_type': 'nt1vm',
            'new_node_type': 'nt2'
        })
        _create_cluster(self, self.kwargs)
        _wait_for_cluster_state_ready(self, self.kwargs)

        # add node type nt2
        self.cmd('az sf cluster node-type add -g {rg} -c {cluster_name} --node-type {new_node_type} --capacity 6 --vm-user-name admintest '
                 '--vm-password {vm_password} --durability-level Bronze --vm-sku Standard_D15_v2',
                 checks=[self.check('provisioningState', 'Succeeded'),
                         self.check('length(nodeTypes)', 2),
                         self.check('nodeTypes[1].name', 'nt2'),
                         self.check('nodeTypes[1].vmInstanceCount', 6),
                         self.check('nodeTypes[1].durabilityLevel', 'Bronze')])

        # remvoe node from node type nt2
        self.cmd('sf cluster node remove -g {rg} -c {cluster_name} --node-type {new_node_type} --number-of-nodes-to-remove 1',
                 checks=[self.check('nodeTypes[1].vmInstanceCount', 5)])

        # update to duribility to Silver of node type nt2
        self.cmd('sf cluster durability update --resource-group {rg} -c {cluster_name} --durability-level Silver --node-type {new_node_type}',
                 checks=[self.check('nodeTypes[1].durabilityLevel', 'Silver')])

        # add setting
        self.cmd('sf cluster setting set --resource-group {rg} -c {cluster_name} --section NamingService --parameter MaxOperationTimeout --value 10001',
                 checks=[self.check('length(fabricSettings)', 2),
                         self.check('fabricSettings[1].name', 'NamingService'),
                         self.check('fabricSettings[1].parameters[0].name', 'MaxOperationTimeout'),
                         self.check('fabricSettings[1].parameters[0].value', '10001')])

        # remove setting
        self.cmd('sf cluster setting remove --resource-group {rg} -c {cluster_name} --section NamingService --parameter MaxOperationTimeout',
                 checks=[self.check('length(fabricSettings)', 1),
                         self.check('fabricSettings[0].name', 'Security')])

        # add node primary node type nt1vm
        self.cmd('sf cluster node add -g {rg} -c {cluster_name} --node-type {priamry_node_type} --number-of-nodes-to-add 2',
                 checks=[self.check('nodeTypes[0].vmInstanceCount', 5)])

        # update reliability to Silver
        self.cmd('sf cluster reliability update --resource-group {rg} -c {cluster_name} --reliability-level Silver',
                 checks=[self.check('reliabilityLevel', 'Silver')])


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
import uuid
import os

from knack.util import CLIError
from azure_functions_devops_build.exceptions import RoleAssignmentException
from azure.cli.testsdk import LiveScenarioTest, ResourceGroupPreparer, StorageAccountPreparer, JMESPathCheck

CURR_DIR = os.getcwd()
TEST_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), 'sample_dotnet_function'))


class DevopsBuildCommandsTest(LiveScenarioTest):
    def setUp(self):
        super().setUp()
        # You must be the organization owner and the subscription owner to run the following tests
        # 1. Install devops extension 'az extension add --name azure-devops'
        # 2. Login with 'az login'
        # 3. Go to dev.azure.com apply for a personal access token, and login with 'az devops login'
        # 4. Change the self.azure_devops_organization to your Azure DevOps organization
        self.azure_devops_organization = "azureclitest"  # Put "<Your DevOps Organization>" to record live tests. Please change back to "azureclitest" as we have a routine live tests pipeline using this account.
        self.os_type = "Windows"
        self.runtime = "dotnet"

        self.functionapp = self.create_random_name(prefix='test-functionapp-', length=24)
        self.azure_devops_project = self.create_random_name(prefix='test-project-', length=24)
        self.azure_devops_repository = self.create_random_name(prefix='test-repository-', length=24)

        self.kwargs.update({
            'ot': self.os_type,
            'rt': self.runtime,
            'org': self.azure_devops_organization,
            'proj': self.azure_devops_project,
            'repo': self.azure_devops_repository,
            'fn': self.functionapp,
        })

    @unittest.skip("test has been failing continuously and functions team needs to fix this.")
    @ResourceGroupPreparer(random_name_length=24)
    @StorageAccountPreparer(parameter_name='storage_account_for_test')
    def test_devops_build_command(self, resource_group, resource_group_location, storage_account_for_test):
        self._setUpDevopsEnvironment(resource_group, resource_group_location, storage_account_for_test)

        # Test devops build command
        try:
            result = self.cmd('functionapp devops-pipeline create --organization-name {org} --project-name {proj}'
                              ' --repository-name {repo} --functionapp-name {fn} --allow-force-push true'
                              ' --overwrite-yaml true').get_output_in_json()

            self.assertEqual(result['functionapp_name'], self.functionapp)
            self.assertEqual(result['organization_name'], self.azure_devops_organization)
            self.assertEqual(result['project_name'], self.azure_devops_project)
            self.assertEqual(result['repository_name'], self.azure_devops_repository)
        except CLIError:
            raise unittest.SkipTest('You must be the owner of the subscription')
        finally:
            self._tearDownDevopsEnvironment()

    @unittest.skip("test has been failing continuously and functions team needs to fix this.")
    @ResourceGroupPreparer(random_name_length=24)
    @StorageAccountPreparer(parameter_name='storage_account_for_test')
    def test_devops_build_mismatch_runtime(self, resource_group, resource_group_location, storage_account_for_test):
        # Overwrite function runtime to use node
        self.kwargs.update({'rt': 'node'})
        self._setUpDevopsEnvironment(resource_group, resource_group_location, storage_account_for_test)

        # Test devops build command (mismatched local_runtime:dotnet vs remote_runtime:node)
        try:
            self.cmd('functionapp devops-pipeline create --organization-name {org} --project-name {proj} '
                     '--repository-name {repo} --functionapp-name {fn} --allow-force-push true '
                     '--overwrite-yaml true', expect_failure=True)
        finally:
            self._tearDownDevopsEnvironment()

    @unittest.skip("test has been failing continuously and functions team needs to fix this.")
    @ResourceGroupPreparer(random_name_length=24)
    @StorageAccountPreparer(parameter_name='storage_account_for_test')
    def test_devops_build_mismatch_functionapp(self, resource_group, resource_group_location, storage_account_for_test):
        self._setUpDevopsEnvironment(resource_group, resource_group_location, storage_account_for_test)

        # Overwrite functionapp name to use a mismatched value
        self.kwargs.update({'fn': self.create_random_name(prefix='mismatch', length=24)})

        try:
            self.cmd('functionapp devops-pipeline create --organization-name {org} --project-name {proj} '
                     '--repository-name {repo} --functionapp-name {fn} --allow-force-push true '
                     '--overwrite-yaml true', expect_failure=True)
        finally:
            self.kwargs.update({'fn': self.functionapp})
            self._tearDownDevopsEnvironment()

    @unittest.skip("test has been failing continuously and functions team needs to fix this.")
    @ResourceGroupPreparer(random_name_length=24)
    @StorageAccountPreparer(parameter_name='storage_account_for_test')
    def test_devops_build_mismatch_organization(self, resource_group, resource_group_location, storage_account_for_test):
        self._setUpDevopsEnvironment(resource_group, resource_group_location, storage_account_for_test)

        # Overwrite organization name to use a mismatched value
        self.kwargs.update({'org': self.create_random_name(prefix='mismatch', length=24)})

        try:
            self.cmd('functionapp devops-pipeline create --organization-name {org} --project-name {proj} '
                     '--repository-name {repo} --functionapp-name {fn} --allow-force-push true '
                     '--overwrite-yaml true', expect_failure=True)
        finally:
            self.kwargs.update({'org': self.azure_devops_organization})
            self._tearDownDevopsEnvironment()

    @unittest.skip("test has been failing continuously and functions team needs to fix this.")
    @ResourceGroupPreparer(random_name_length=24)
    @StorageAccountPreparer(parameter_name='storage_account_for_test')
    def test_devops_build_mismatch_project(self, resource_group, resource_group_location, storage_account_for_test):
        self._setUpDevopsEnvironment(resource_group, resource_group_location, storage_account_for_test)

        # Overwrite project name to use a mismatched value
        self.kwargs.update({'proj': self.create_random_name(prefix='mismatch', length=24)})

        try:
            self.cmd('functionapp devops-pipeline create --organization-name {org} --project-name {proj} '
                     '--repository-name {repo} --functionapp-name {fn} --allow-force-push true '
                     '--overwrite-yaml true', expect_failure=True)
        finally:
            self.kwargs.update({'proj': self.azure_devops_project})
            self._tearDownDevopsEnvironment()

    # Devops environment utilities
    def _setUpDevopsEnvironment(self, resource_group, resource_group_location, storage_account_for_test):
        self.kwargs.update({
            'rg': resource_group,
            'cpl': resource_group_location,
            'sa': storage_account_for_test,
        })

        # Create a new functionapp
        self.cmd('functionapp create --resource-group {rg} --storage-account {sa} '
                 '--os-type {ot} --runtime {rt} --name {fn} --consumption-plan-location {cpl}',
                 checks=[JMESPathCheck('name', self.functionapp), JMESPathCheck('resourceGroup', resource_group)]
                 )

        # Install azure devops extension
        self.cmd('extension add --name azure-devops')

        # Create a new project in Azure Devops
        result = self.cmd('devops project create --organization https://dev.azure.com/{org} --name {proj}', checks=[
            JMESPathCheck('name', self.azure_devops_project),
        ]).get_output_in_json()
        self.azure_devops_project_id = result['id']

        # Create a new repository in Azure Devops
        self.cmd('repos create --organization https://dev.azure.com/{org} --project {proj} --name {repo}', checks=[
            JMESPathCheck('name', self.azure_devops_repository),
        ])

        # Change directory to sample functionapp
        os.chdir(TEST_DIR)

    def _tearDownDevopsEnvironment(self):
        import time
        # Change directory back
        os.chdir(CURR_DIR)

        # Remove Azure Devops project
        retry = 5
        for i in range(retry):
            try:
                self.cmd('devops project delete --organization https://dev.azure.com/{org} --id {id} --yes'.format(
                    org=self.azure_devops_organization,
                    id=self.azure_devops_project_id
                ))
                break
            except Exception as ex:
                if i == retry - 1:
                    raise ex
                time.sleep(120)



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines

helps['relay'] = """
type: group
short-summary: Manage Azure Relay Service namespaces, WCF relays, hybrid connections, and rules
"""

helps['relay hyco'] = """
type: group
short-summary: Manage Azure Relay Service Hybrid Connection and Authorization Rule
"""

helps['relay hyco authorization-rule'] = """
type: group
short-summary: Manage Azure Relay Service Hybrid Connection Authorization Rule
"""

helps['relay hyco authorization-rule create'] = """
type: command
short-summary: Create Authorization Rule for given Relay Service Hybrid Connection
examples:
  - name: Create Authorization Rule for given Relay Service Hybrid Connection
    text: az relay hyco authorization-rule create --resource-group myresourcegroup --namespace-name mynamespace --hybrid-connection-name myhyco --name myauthorule --rights Send Listen
"""

helps['relay hyco authorization-rule delete'] = """
type: command
short-summary: Deletes the Authorization Rule of the given Relay Service Hybrid Connection.
examples:
  - name: Deletes the Authorization Rule of Relay Service Hybrid Connection.
    text: az relay hyco authorization-rule delete --resource-group myresourcegroup --namespace-name mynamespace --hybrid-connection-name myhyco --name myauthorule
"""

helps['relay hyco authorization-rule keys'] = """
type: group
short-summary: Manage Azure Authorization Rule keys for Relay Service Hybrid Connection
"""

helps['relay hyco authorization-rule keys list'] = """
type: command
short-summary: List the keys and connection strings of Authorization Rule for Relay Service Hybrid Connection.
examples:
  - name: List the keys and connection strings of Authorization Rule for Relay Service Hybrid Connection.
    text: az relay hyco authorization-rule keys list --resource-group myresourcegroup --namespace-name mynamespace --hybrid-connection-name myhyco --name myauthorule
  - name: List the keys and connection strings of Authorization Rule for Relay Service Hybrid Connection (autogenerated)
    text: az relay hyco authorization-rule keys list --hybrid-connection-name myhyco --name myauthorule --namespace-name mynamespace --resource-group myresourcegroup --subscription MySubscription
    crafted: true
"""

helps['relay hyco authorization-rule keys renew'] = """
type: command
short-summary: Regenerate keys of Authorization Rule for Relay Service Hybrid Connection.
examples:
  - name: Regenerate key of Relay Service Hybrid Connection.
    text: az relay hyco authorization-rule keys renew --resource-group myresourcegroup --namespace-name mynamespace --hybrid-connection-name myhyco --name myauthorule --key PrimaryKey
"""

helps['relay hyco authorization-rule list'] = """
type: command
short-summary: shows list of Authorization Rule by Relay Service Hybrid Connection
examples:
  - name: shows list of Authorization Rule by Relay Service Hybrid Connection
    text: az relay hyco authorization-rule list --resource-group myresourcegroup --namespace-name mynamespace --hybrid-connection-name myhyco
"""

helps['relay hyco authorization-rule show'] = """
type: command
short-summary: Shows the details of Authorization Rule for given Relay Service Hybrid Connection
examples:
  - name: Shows the details of Authorization Rule for given Relay Service Hybrid Connection
    text: az relay hyco authorization-rule show --resource-group myresourcegroup --namespace-name mynamespace --hybrid-connection-name myhyco --name myauthorule
"""

helps['relay hyco authorization-rule update'] = """
type: command
short-summary: Create Authorization Rule for given Relay Service Hybrid Connection
examples:
  - name: Create Authorization Rule for given Relay Service Hybrid Connection
    text: az relay hyco authorization-rule update --resource-group myresourcegroup --namespace-name mynamespace --hybrid-connection-name myhyco --name myauthorule --rights Send
"""

helps['relay hyco create'] = """
type: command
short-summary: Create the Relay Service Hybrid Connection
examples:
  - name: Create a new Relay Service Hybrid Connection
    text: az relay hyco create --resource-group myresourcegroup --namespace-name mynamespace --name myhyco
"""

helps['relay hyco delete'] = """
type: command
short-summary: Deletes the Relay Service Hybrid Connection
examples:
  - name: Deletes the Relay Service Hybrid Connection
    text: az relay hyco delete --resource-group myresourcegroup --namespace-name mynamespace --name myhyco
"""

helps['relay hyco list'] = """
type: command
short-summary: List the Hybrid Connection by Relay Service Namepsace
examples:
  - name: Get the Hybrid Connections by Namespace.
    text: az relay hyco list --resource-group myresourcegroup --namespace-name mynamespace
"""

helps['relay hyco show'] = """
type: command
short-summary: Shows the Relay Service Hybrid Connection Details
examples:
  - name: Shows the Hybrid Connection details.
    text: az relay hyco show --resource-group myresourcegroup --namespace-name mynamespace --name myhyco
"""

helps['relay hyco update'] = """
type: command
short-summary: Updates the Relay Service Hybrid Connection
examples:
  - name: Updates existing Relay Service Hybrid Connection.
    text: az relay hyco update --resource-group myresourcegroup --namespace-name mynamespace --name myhyco
"""

helps['relay namespace'] = """
type: group
short-summary: Manage Azure Relay Service Namespace
"""

helps['relay namespace authorization-rule'] = """
type: group
short-summary: Manage Azure Relay Service Namespace Authorization Rule
"""

helps['relay namespace authorization-rule create'] = """
type: command
short-summary: Create Authorization Rule for the given Relay Service Namespace
examples:
  - name: Create Authorization Rule 'myrule' for the given Relay Service Namespace 'mynamespace' in resourcegroup
    text: az relay namespace authorization-rule create --resource-group myresourcegroup --namespace-name mynamespace --name myauthorule --rights Send Listen
"""

helps['relay namespace authorization-rule delete'] = """
type: command
short-summary: Deletes the Authorization Rule of the Relay Service Namespace.
examples:
  - name: Deletes the Authorization Rule of the Relay Service Namespace.
    text: az relay namespace authorization-rule delete --resource-group myresourcegroup --namespace-name mynamespace --name myauthorule
"""

helps['relay namespace authorization-rule keys'] = """
type: group
short-summary: Manage Azure Authorization Rule connection strings for Namespace
"""

helps['relay namespace authorization-rule keys list'] = """
type: command
short-summary: List the keys and connection strings of Authorization Rule for Relay Service Namespace
examples:
  - name: List the keys and connection strings of Authorization Rule for Relay Service Namespace
    text: az relay namespace authorization-rule keys list --resource-group myresourcegroup --namespace-name mynamespace --name myauthorule
  - name: List the keys and connection strings of Authorization Rule for Relay Service Namespace (commonly used with --output and --query). (autogenerated)
    text: az relay namespace authorization-rule keys list --name myauthorule --namespace-name mynamespace --resource-group myresourcegroup --subscription MySubscription
    crafted: true
"""

helps['relay namespace authorization-rule keys renew'] = """
type: command
short-summary: Regenerate keys of Authorization Rule for the Relay Service Namespace.
examples:
  - name: Regenerate keys of Authorization Rule for the Relay Service Namespace.
    text: az relay namespace authorization-rule keys renew --resource-group myresourcegroup --namespace-name mynamespace --name myauthorule --key PrimaryKey
"""

helps['relay namespace authorization-rule list'] = """
type: command
short-summary: Shows the list of Authorization Rule by Relay Service Namespace
examples:
  - name: Shows the list of Authorization Rule by Relay Service Namespace
    text: az relay namespace authorization-rule list --resource-group myresourcegroup --namespace-name mynamespace
"""

helps['relay namespace authorization-rule show'] = """
type: command
short-summary: Shows the details of Relay Service Namespace Authorization Rule
examples:
  - name: Shows the details of Relay Service Namespace Authorization Rule
    text: az relay namespace authorization-rule show --resource-group myresourcegroup --namespace-name mynamespace --name myauthorule
"""

helps['relay namespace authorization-rule update'] = """
type: command
short-summary: Updates Authorization Rule for the given Relay Service Namespace
examples:
  - name: Updates Authorization Rule 'myrule' for the given Relay Service Namespace 'mynamespace' in resourcegroup
    text: az relay namespace authorization-rule update --resource-group myresourcegroup --namespace-name mynamespace --name myauthorule --rights Send
"""

helps['relay namespace create'] = """
type: command
short-summary: Create a Relay Service Namespace
examples:
  - name: Create a Relay Service Namespace.
    text: az relay namespace create --resource-group myresourcegroup --name mynamespace --location westus --tags tag1=value1 tag2=value2
"""

helps['relay namespace delete'] = """
type: command
short-summary: Deletes the Relay Service Namespace
examples:
  - name: Deletes the Relay Service Namespace
    text: az relay namespace delete --resource-group myresourcegroup --name mynamespace
"""

helps['relay namespace exists'] = """
type: command
short-summary: check for the availability of the given name for the Namespace
examples:
  - name: check for the availability of mynamespace for the Namespace
    text: az relay namespace exists --name mynamespace
"""

helps['relay namespace list'] = """
type: command
short-summary: List the Relay Service Namespaces
examples:
  - name: Get the Relay Service Namespaces by resource group
    text: az relay namespace list --resource-group myresourcegroup
  - name: Get the Relay Service Namespaces by Subscription.
    text: az relay namespace list
"""

helps['relay namespace show'] = """
type: command
short-summary: Shows the Relay Service Namespace details
examples:
  - name: shows the Namespace details.
    text: az relay namespace show --resource-group myresourcegroup --name mynamespace
"""

helps['relay namespace update'] = """
type: command
short-summary: Updates a Relay Service Namespace
examples:
  - name: Updates a Relay Service Namespace.
    text: az relay namespace update --resource-group myresourcegroup --name mynamespace --tags tag=value
"""

helps['relay wcfrelay'] = """
type: group
short-summary: Manage Azure Relay Service WCF Relay and Authorization Rule
"""

helps['relay wcfrelay authorization-rule'] = """
type: group
short-summary: Manage Azure Relay Service WCF Relay Authorization Rule
"""

helps['relay wcfrelay authorization-rule create'] = """
type: command
short-summary: Create Authorization Rule for the given Relay Service WCF Relay.
examples:
  - name: Create Authorization Rule for WCF Relay
    text: az relay wcfrelay authorization-rule create --resource-group myresourcegroup --namespace-name mynamespace --relay-name myrelay --name myauthorule --rights Listen
"""

helps['relay wcfrelay authorization-rule delete'] = """
type: command
short-summary: Delete the Authorization Rule of Relay Service WCF Relay
examples:
  - name: Delete the Authorization Rule of Relay Service WCF Relay
    text: az relay wcfrelay authorization-rule delete --resource-group myresourcegroup --namespace-name mynamespace --relay-name myrelay --name myauthorule
"""

helps['relay wcfrelay authorization-rule keys'] = """
type: group
short-summary: Manage Azure Authorization Rule keys for Relay Service WCF Relay
"""

helps['relay wcfrelay authorization-rule keys list'] = """
type: command
short-summary: List the keys and connection strings of Authorization Rule for the given Relay Service WCF Relay
examples:
  - name: List the keys and connection strings of Authorization Rule for the given Relay Service WCF Relay
    text: az relay wcfrelay authorization-rule keys list --resource-group myresourcegroup --namespace-name mynamespace --relay-name myrelay --name myauthorule
"""

helps['relay wcfrelay authorization-rule keys renew'] = """
type: command
short-summary: Regenerate keys of Authorization Rule for Relay Service WCF Relay
examples:
  - name: Regenerate keys of Authorization Rule for Relay Service WCF Relay
    text: az relay wcfrelay authorization-rule keys renew --resource-group myresourcegroup --namespace-name mynamespace --relay-name myrelay --name myauthorule --key PrimaryKey
"""

helps['relay wcfrelay authorization-rule list'] = """
type: command
short-summary: List of Authorization Rule by Relay Service WCF Relay.
examples:
  - name: List of Authorization Rule by WCF Relay
    text: az relay wcfrelay authorization-rule list --resource-group myresourcegroup --namespace-name mynamespace --relay-name myrelay
"""

helps['relay wcfrelay authorization-rule show'] = """
type: command
short-summary: show properties of Authorization Rule for the given Relay Service WCF Relay.
examples:
  - name: show properties of Authorization Rule
    text: az relay wcfrelay authorization-rule show --resource-group myresourcegroup --namespace-name mynamespace --relay-name myrelay --name myauthorule
"""

helps['relay wcfrelay authorization-rule update'] = """
type: command
short-summary: Update Authorization Rule for the given Relay Service WCF Relay.
examples:
  - name: Update Authorization Rule for WCF Relay
    text: az relay wcfrelay authorization-rule update --resource-group myresourcegroup --namespace-name mynamespace --relay-name myrelay --name myauthorule --rights Send
"""

helps['relay wcfrelay create'] = """
type: command
short-summary: Create the Relay Service WCF Relay
examples:
  - name: Create Relay Service WCF Relay.
    text: az relay wcfrelay create --resource-group myresourcegroup --namespace-name mynamespace --name myrelay --relay-type NetTcp
"""

helps['relay wcfrelay delete'] = """
type: command
short-summary: Deletes the Relay Service WCF Relay
examples:
  - name: Deletes the wcfrelay
    text: az relay wcfrelay delete --resource-group myresourcegroup --namespace-name mynamespace --name myrelay
"""

helps['relay wcfrelay list'] = """
type: command
short-summary: List the WCF Relay by Relay Service Namepsace
examples:
  - name: Get the WCF Relays by Relay Service Namespace.
    text: az relay wcfrelay list --resource-group myresourcegroup --namespace-name mynamespace
"""

helps['relay wcfrelay show'] = """
type: command
short-summary: shows the Relay Service WCF Relay Details
examples:
  - name: Shows the Relay Service WCF Relay Details
    text: az relay wcfrelay show --resource-group myresourcegroup --namespace-name mynamespace --name myrelay
"""

helps['relay wcfrelay update'] = """
type: command
short-summary: Updates existing Relay Service WCF Relay
examples:
  - name: Updates Relay Service WCF Relay.
    text: az relay wcfrelay update --resource-group myresourcegroup --namespace-name mynamespace --name myrelay
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import argparse
import sys
import automation.verify
import automation.style
import automation.tests
import automation.cli_linter


def main():
    parser = argparse.ArgumentParser(prog='azdev')

    sub_parser = parser.add_subparsers(title='sub commands')
    automation.verify.init_args(sub_parser)
    automation.style.init_args(sub_parser)
    automation.tests.init_args(sub_parser)
    automation.cli_linter.init_args(sub_parser)

    if sys.argv[1:]:
        args = parser.parse_args()
        args.func(args)
    else:
        parser.print_help()


if __name__ == '__main__':
    main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer, JMESPathCheck, NoneCheck,
                               api_version_constraint)
from azure.cli.core.profiles import ResourceType
from ..storage_test_util import StorageScenarioMixin


@api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2016-12-01')
class StorageAccessControlListTests(StorageScenarioMixin, ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_container_acl_scenarios(self, resource_group, storage_account):
        account_info = self.get_account_info(resource_group, storage_account)
        container = self.create_container(account_info)

        self._verify_access_control_list(account_info, 'container', container)

    # @ResourceGroupPreparer()
    # @StorageAccountPreparer()
    # def test_storage_share_acl_scenarios(self, resource_group, storage_account):
    #     account_info = self.get_account_info(resource_group, storage_account)
    #     share = self.create_share(account_info)

    #     self._verify_access_control_list(account_info, 'share', share)

    def _verify_access_control_list(self, account_info, service_type, container_name):
        container_id_parameter = '--{}-name {}'.format(service_type, container_name)
        self.storage_cmd('storage {} policy list {}', account_info, service_type,
                         container_id_parameter).assert_with_checks(NoneCheck())
        self.storage_cmd('storage {} policy create {} -n test1 --permission l', account_info, service_type,
                         container_id_parameter)
        self.storage_cmd('storage {} policy create {} -n test2 --start 2016-01-01T00:00Z', account_info, service_type,
                         container_id_parameter)
        self.storage_cmd('storage {} policy create {} -n test3 --expiry 2018-01-01T00:00Z', account_info, service_type,
                         container_id_parameter)
        self.storage_cmd('storage {} policy create {} -n test4 --permission rwdl '
                         '--start 2016-01-01T00:00Z --expiry 2016-05-01T00:00Z', account_info, service_type,
                         container_id_parameter)

        acl = self.storage_cmd('storage {} policy list {}', account_info, service_type,
                               container_id_parameter).get_output_in_json().keys()

        self.assertSetEqual(set(acl), set(['test1', 'test2', 'test3', 'test4']))

        self.storage_cmd('storage {} policy show {} -n test1', account_info, service_type,
                         container_id_parameter).assert_with_checks(JMESPathCheck('permission', 'l'))
        self.storage_cmd('storage {} policy show {} -n test2', account_info, service_type,
                         container_id_parameter).assert_with_checks(JMESPathCheck('start', '2016-01-01T00:00:00+00:00'))
        self.storage_cmd('storage {} policy show {} -n test3', account_info, service_type,
                         container_id_parameter).assert_with_checks(
                             JMESPathCheck('expiry', '2018-01-01T00:00:00+00:00'))
        self.storage_cmd('storage {} policy show {} -n test4', account_info, service_type,
                         container_id_parameter).assert_with_checks(JMESPathCheck('start', '2016-01-01T00:00:00+00:00'),
                                                                    JMESPathCheck('expiry',
                                                                                  '2016-05-01T00:00:00+00:00'),
                                                                    JMESPathCheck('permission', 'rwdl'))
        self.storage_cmd('storage {} policy update {} -n test1 --permission r', account_info, service_type,
                         container_id_parameter)
        self.storage_cmd('storage {} policy show {} -n test1', account_info, service_type,
                         container_id_parameter).assert_with_checks(JMESPathCheck('permission', 'r'))
        self.storage_cmd('storage {} policy delete {} -n test1', account_info, service_type, container_id_parameter)

        acl = self.storage_cmd('storage {} policy list {}', account_info, service_type,
                               container_id_parameter).get_output_in_json().keys()

        self.assertSequenceEqual(set(acl), set(['test2', 'test3', 'test4']))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

"""Verify all commands and arguments are loaded without errors. """


FAILED_TO_LOAD = []
EXTENSION_FAILURE_EXCLUSIONS = []

def init(root):
    parser = root.add_parser('load-all', help='Load the full command table, command arguments and help.')
    parser.set_defaults(func=verify_load_all)


def verify_load_all(_):
    from azure.cli.core import get_default_cli, EVENT_FAILED_EXTENSION_LOAD
    from azure.cli.core.file_util import get_all_help, create_invoker_and_load_cmds_and_args

    print('Loading all commands, arguments, and help...')
    # setup CLI to enable command loader and register event
    az_cli = get_default_cli()
    az_cli.register_event(EVENT_FAILED_EXTENSION_LOAD, extension_failed_load_handler)

    # load commands, args, and help
    create_invoker_and_load_cmds_and_args(az_cli)
    loaded_help = get_all_help(az_cli)

    # verify each installed extension is properly loaded
    if not FAILED_TO_LOAD or set(FAILED_TO_LOAD).issubset(set(EXTENSION_FAILURE_EXCLUSIONS)):
        print('Everything loaded successfully.')
    else:
        raise Exception('Exceptions failed to load: {}'.format(', '.join(FAILED_TO_LOAD)))

def extension_failed_load_handler(_, **kwargs):
    FAILED_TO_LOAD.append(kwargs.get('extension_name'))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# Invoke this script in Powershell with:
# & 'C:\Program Files (x86)\Microsoft SDKs\Azure\CLI2\python.exe' test_msi_package.py <start_mod_name>

import os 
import sys
import subprocess 

base_dir = 'C:\\Program Files (x86)\\Microsoft SDKs\\Azure\\CLI2\\Lib\\site-packages\\azure\\cli'
root_dir = '{}\\command_modules'.format(base_dir)
mod_list = [mod for mod in sorted(os.listdir(root_dir)) if os.path.isdir(os.path.join(root_dir, mod)) and mod != '__pycache__']

pytest_base_cmd = ['python', '-m', 'pytest', '-x', '-v', '-p', 'no:warnings', '--log-level', 'WARN']
pytest_parallel_cmd = pytest_base_cmd + ['-n', 'auto']

for mod_name in mod_list:
    try:
        start_mod = sys.argv[1]
        if mod_name < start_mod:
            continue
    except:
        pass
    mod_cmd = ['--junit-xml', '{}\\azure_cli_test_result\\{}.xml'.format(os.path.expanduser('~'), mod_name),
               '--pyargs', 'azure.cli.command_modules.{}'.format(mod_name)]
    if mod_name in ['botservice', 'network']:
        exit_code = subprocess.call(pytest_base_cmd + mod_cmd)
    else:
        exit_code = subprocess.call(pytest_parallel_cmd + mod_cmd)
    if exit_code == 5:
        print('No tests found for {}'.format(mod_name))
    elif exit_code != 0:
        sys.exit(exit_code)

core_dir = '{}\\core'.format(base_dir)
exit_code = subprocess.call(['python', '-m', 'pytest', '-x', '-v', '-p', 'no:warnings', '--log-level', 'WARN', 
    '--junit-xml', '{}\\azure_cli_test_result\\azure-cli-core.xml'.format(os.path.expanduser('~')), '-n', 'auto', '--import-mode=append', core_dir])
sys.exit(exit_code)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import time
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer, api_version_constraint,
                               JMESPathCheck, JMESPathCheckExists, NoneCheck)
from azure.cli.core.profiles import ResourceType


class StorageQueueScenarioTests(ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer(sku='Standard_RAGRS')
    def test_storage_queue_general_scenario(self, resource_group, storage_account):
        account_key = self.get_account_key(resource_group, storage_account)

        self.set_env('AZURE_STORAGE_ACCOUNT', storage_account)
        self.set_env('AZURE_STORAGE_KEY', account_key)

        queue = self.create_random_name('queue', 24)

        self.cmd('storage queue create -n {} --fail-on-exist --metadata a=b c=d'.format(queue),
                 checks=JMESPathCheck('created', True))
        self.cmd('storage queue exists -n {}'.format(queue),
                 checks=JMESPathCheck('exists', True))

        res = self.cmd('storage queue list').get_output_in_json()
        self.assertIn(queue, [x['name'] for x in res], 'The newly created queue is not listed.')

        from datetime import datetime, timedelta
        expiry = (datetime.utcnow() + timedelta(hours=1)).strftime('%Y-%m-%dT%H:%MZ')
        sas = self.cmd('storage queue generate-sas -n {} --permissions r --expiry {}'.format(queue, expiry)).output
        self.assertIn('sig', sas, 'The sig segment is not in the sas {}'.format(sas))

        self.cmd('storage queue metadata show -n {}'.format(queue), checks=[
            JMESPathCheck('a', 'b'),
            JMESPathCheck('c', 'd')
        ])

        self.cmd('storage queue metadata update -n {} --metadata e=f g=h'.format(queue))
        self.cmd('storage queue metadata show -n {}'.format(queue), checks=[
            JMESPathCheck('e', 'f'),
            JMESPathCheck('g', 'h')
        ])

        # Queue message operation
        self.cmd('storage message put -q {} --content "test message"'.format(queue))
        self.cmd('storage message peek -q {}'.format(queue),
                 checks=JMESPathCheck('[0].content', 'test message'))

        first_message = self.cmd('storage message get -q {}'.format(queue),
                                 checks=JMESPathCheck('length(@)', 1)).get_output_in_json()[0]

        self.cmd('storage message update -q {} --id {} --pop-receipt {} --visibility-timeout 1 '
                 '--content "new message!"'.format(queue, first_message['id'],
                                                   first_message['popReceipt']))

        time.sleep(2)  # ensures message should be back in queue

        self.cmd('storage message peek -q {}'.format(queue),
                 checks=JMESPathCheck('[0].content', 'new message!'))
        self.cmd('storage message put -q {} --content "second message"'.format(queue))
        self.cmd('storage message put -q {} --content "third message"'.format(queue))
        self.cmd('storage message peek -q {} --num-messages 32'.format(queue),
                 checks=JMESPathCheck('length(@)', 3))

        third_message = self.cmd('storage message get -q {}'.format(queue)).get_output_in_json()[0]

        self.cmd('storage message delete -q {} --id {} --pop-receipt {}'
                 .format(queue, third_message['id'], third_message['popReceipt']))
        self.cmd('storage message peek -q {} --num-messages 32'.format(queue),
                 checks=JMESPathCheck('length(@)', 2))

        self.cmd('storage message clear -q {}'.format(queue))
        self.cmd('storage message peek -q {} --num-messages 32'.format(queue), checks=NoneCheck())

        # verify delete operation
        self.cmd('storage queue delete -n {} --fail-not-exist'.format(queue),
                 checks=JMESPathCheck('deleted', True))
        self.cmd('storage queue exists -n {}'.format(queue),
                 checks=JMESPathCheck('exists', False))

        # check status of the queue
        queue_status = self.cmd('storage queue stats').get_output_in_json()
        self.assertIn(queue_status['geoReplication']['status'], ('live', 'unavailable'))

    def get_account_key(self, group, name):
        return self.cmd('storage account keys list -n {} -g {} --query "[0].value" -otsv'
                        .format(name, group)).output


if __name__ == '__main__':
    import unittest

    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

"""Verify the list of modules that should be included as part of the CLI install. """

from __future__ import print_function

import os
import sys
import json
import tempfile
import zipfile
import glob

from automation.utilities.path import (get_repo_root, get_command_modules_paths)
from automation.utilities.display import print_heading

AZURE_CLI_PATH = os.path.join(get_repo_root(), 'src', 'azure-cli')
AZURE_CLI_SETUP_PY = os.path.join(AZURE_CLI_PATH, 'setup.py')


def get_cli_dependencies(build_folder):
    azure_cli_wheel = glob.glob(build_folder.rstrip('/') + '/azure_cli-*.whl')[0]
    print('Explore wheel file {}.'.format(azure_cli_wheel))

    tmp_dir = tempfile.mkdtemp()

    zip_ref = zipfile.ZipFile(azure_cli_wheel, 'r')
    zip_ref.extractall(tmp_dir)
    zip_ref.close()

    dist_info_dir = [f for f in os.listdir(tmp_dir) if f.endswith('.dist-info')][0]
    whl_metadata_filepath = os.path.join(tmp_dir, dist_info_dir, 'metadata.json')
    with open(whl_metadata_filepath) as f:
        print('Load metadata file from {}'.format(whl_metadata_filepath))
        return json.load(f)['run_requires'][0]['requires']


def verify_default_modules(args):
    errors_list = []
    cli_deps = get_cli_dependencies(args.build_folder)
    all_command_modules = get_command_modules_paths(include_prefix=True)
    if not cli_deps:
        print('Unable to get the CLI dependencies for {}'.format(AZURE_CLI_SETUP_PY), file=sys.stderr)
        sys.exit(1)
    for modname, _ in all_command_modules:
        if modname not in cli_deps:
            errors_list.append("{} is not included to be installed by default! Modify {}.".format(modname, AZURE_CLI_SETUP_PY))
    if errors_list:
        print_heading('Errors whilst verifying default modules list in {}!'.format(AZURE_CLI_SETUP_PY))
        print('\n'.join(errors_list), file=sys.stderr)
        sys.exit(1)
    else:
        print('Verified default modules list successfully.', file=sys.stderr)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long

from azure_devtools.scenario_tests import RecordingProcessor


class ExpressRoutePortLOAContentReplacer(RecordingProcessor):

    def process_response(self, response):
        import json
        import base64

        body = response['body']['string']

        json_body = json.loads(body)
        json_body['encodedContent'] = base64.b64encode('Raw bytes of a PDF content replaced by ExpressRoutePortLOAContentReplacer'.encode('utf-8')).decode('utf-8')

        response['body']['string'] = json.dumps(json_body)

        return response



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from ..rule_decorators import command_rule
from ..linter import RuleError


@command_rule
def missing_command_help(linter, command_name):
    if not linter.get_command_help(command_name) and not linter.command_expired(command_name):
        raise RuleError('Missing help')


@command_rule
def no_ids_for_list_commands(linter, command_name):
    if command_name.split()[-1] == 'list' and 'ids' in linter.get_command_parameters(command_name):
        raise RuleError('List commands should not expose --ids argument')

@command_rule
def expired_command(linter, command_name):
    if linter.command_expired(command_name):
        raise RuleError("Deprecated command is expired and should be removed.".format(command_name))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import CliCommandType

from ._client_factory import cf_configstore, cf_configstore_operations
from ._format import (configstore_credential_format,
                      configstore_identity_format,
                      configstore_output_format,
                      keyvalue_entry_format,
                      featureflag_entry_format,
                      featurefilter_entry_format)


def load_command_table(self, _):

    configstore_custom_util = CliCommandType(
        operations_tmpl='azure.cli.command_modules.appconfig.custom#{}',
        table_transformer=configstore_output_format,
        client_factory=cf_configstore
    )

    configstore_identity_util = CliCommandType(
        operations_tmpl='azure.cli.command_modules.appconfig.custom#{}',
        table_transformer=configstore_identity_format,
        client_factory=cf_configstore
    )

    configstore_credential_util = CliCommandType(
        operations_tmpl='azure.cli.command_modules.appconfig.custom#{}',
        table_transformer=configstore_credential_format,
        client_factory=cf_configstore
    )

    configstore_keyvalue_util = CliCommandType(
        operations_tmpl='azure.cli.command_modules.appconfig.keyvalue#{}',
        table_transformer=keyvalue_entry_format,
        client_factory=cf_configstore_operations
    )

    def get_custom_sdk(custom_module, client_factory, table_transformer):
        """Returns a CliCommandType instance with specified operation template based on the given custom module name.
        This is useful when the command is not defined in the default 'custom' module but instead in a module under
        'operations' package."""
        return CliCommandType(
            operations_tmpl='azure.cli.command_modules.appconfig.{}#'.format(custom_module) + '{}',
            client_factory=client_factory,
            table_transformer=table_transformer
        )

    # Management Plane Commands
    with self.command_group('appconfig', configstore_custom_util) as g:
        g.command('create', 'create_configstore')
        g.command('delete', 'delete_configstore')
        g.command('update', 'update_configstore')
        g.command('list', 'list_configstore')
        g.show_command('show', 'show_configstore')

    with self.command_group('appconfig credential', configstore_credential_util) as g:
        g.command('list', 'list_credential')
        g.command('regenerate', 'regenerate_credential')

    with self.command_group('appconfig identity', configstore_identity_util, is_preview=True) as g:
        g.command('assign', 'assign_managed_identity')
        g.command('remove', 'remove_managed_identity')
        g.show_command('show', 'show_managed_identity')

    with self.command_group('appconfig revision', configstore_keyvalue_util) as g:
        g.command('list', 'list_revision')

    # Data Plane Commands
    with self.command_group('appconfig kv', configstore_keyvalue_util) as g:
        g.command('set', 'set_key')
        g.command('delete', 'delete_key')
        g.show_command('show', 'show_key')
        g.command('list', 'list_key')
        g.command('lock', 'lock_key')
        g.command('unlock', 'unlock_key')
        g.command('restore', 'restore_key')
        g.command('import', 'import_config')
        g.command('export', 'export_config')
        g.command('set-keyvault', 'set_keyvault')

    # FeatureManagement Commands
    with self.command_group('appconfig feature',
                            custom_command_type=get_custom_sdk('feature',
                                                               cf_configstore_operations,
                                                               featureflag_entry_format),
                            is_preview=True) as g:
        g.custom_command('set', 'set_feature')
        g.custom_command('delete', 'delete_feature')
        g.custom_show_command('show', 'show_feature')
        g.custom_command('list', 'list_feature')
        g.custom_command('lock', 'lock_feature')
        g.custom_command('unlock', 'unlock_feature')
        g.custom_command('enable', 'enable_feature')
        g.custom_command('disable', 'disable_feature')

    with self.command_group('appconfig feature filter',
                            custom_command_type=get_custom_sdk('feature',
                                                               cf_configstore_operations,
                                                               featurefilter_entry_format)) as g:
        g.custom_command('add', 'add_filter')
        g.custom_command('delete', 'delete_filter')
        g.custom_show_command('show', 'show_filter')
        g.custom_command('list', 'list_filter')
